<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

use Migrations\AbstractMigration;

/**
 * Class Initial
 *
 * Created via:
 *  oitc4 migrations create -p AutoreportModule Initial
 *
 * Run migration:
 * oitc4 migrations migrate -p AutoreportModule
 *
 */
class Initial extends AbstractMigration {
    /**
     * Whether the tables created in this migration
     * should auto-create an `id` field or not
     *
     * This option is global for all tables created in the migration file.
     * If you set it to false, you have to manually add the primary keys for your
     * tables using the Migrations\Table::addPrimaryKey() method
     *
     * @var bool
     */
    public $autoId = false;

    /**
     * Up Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-up-method
     * @return void
     */
    public function up(): void {
        if (!$this->hasTable('autoreport_quarterlies')) {
            $this->table('autoreport_quarterlies')
                ->addColumn('id', 'integer', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 11,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('year', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('quarter', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('filename', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])
                ->addColumn('created', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->create();
        }

        if (!$this->hasTable('autoreport_settings')) {
            $this->table('autoreport_settings')
                ->addColumn('id', 'integer', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 11,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('configuration_option', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])
                ->addColumn('store_path', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])
                ->addColumn('notification_emails', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])
                ->addColumn('created', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addColumn('modified', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->create();
        }

        if (!$this->hasTable('autoreports')) {
            $this->table('autoreports')
                ->addColumn('id', 'integer', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 11,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('name', 'string', [
                    'default' => null,
                    'limit'   => 200,
                    'null'    => false,
                ])
                ->addColumn('description', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])
                ->addColumn('container_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('timeperiod_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('report_interval', 'string', [
                    'default' => null,
                    'limit'   => 20,
                    'null'    => false,
                ])
                ->addColumn('report_send_interval', 'string', [
                    'default' => null,
                    'limit'   => 20,
                    'null'    => false,
                ])
                ->addColumn('consider_downtimes', 'boolean', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addColumn('last_send_date', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addColumn('min_availability_percent', 'boolean', [
                    'default' => true,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addColumn('min_availability', 'float', [
                    'default'   => null,
                    'null'      => true,
                    'precision' => 8,
                    'scale'     => 3,
                ])
                ->addColumn('check_hard_state', 'boolean', [
                    'default' => false,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addColumn('use_start_time', 'integer', [
                    'default' => '0',
                    'limit'   => 4,
                    'null'    => false,
                ])
                ->addColumn('report_start_date', 'date', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => true,
                ])
                ->addColumn('last_percent_value', 'float', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addColumn('last_absolut_value', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('show_time', 'integer', [
                    'default' => '1',
                    'limit'   => 4,
                    'null'    => false,
                ])
                ->addColumn('last_number_of_outages', 'integer', [
                    'default' => '0',
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('failure_statistic', 'integer', [
                    'default' => '0',
                    'limit'   => 4,
                    'null'    => false,
                ])
                ->addColumn('consider_holidays', 'integer', [
                    'default' => '0',
                    'limit'   => 4,
                    'null'    => false,
                ])
                ->addColumn('calendar_id', 'integer', [
                    'default' => '0',
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('max_number_of_outages', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => true,
                ])
                ->addColumn('created', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addColumn('modified', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->create();
        }

        if (!$this->hasTable('autoreports_availability_log')) {
            $this->table('autoreports_availability_log')
                ->addColumn('id', 'biginteger', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 20,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('autoreport_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('report_interval', 'string', [
                    'default' => null,
                    'limit'   => 20,
                    'null'    => false,
                ])
                ->addColumn('last_update', 'biginteger', [
                    'default' => null,
                    'limit'   => 20,
                    'null'    => false,
                ])
                ->addColumn('evaluation_start', 'biginteger', [
                    'default' => null,
                    'limit'   => 20,
                    'null'    => false,
                ])
                ->addColumn('evaluation_end', 'biginteger', [
                    'default' => null,
                    'limit'   => 20,
                    'null'    => false,
                ])
                ->addColumn('availability_total_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('minimal_availability_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('determined_availability_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('determined_outage_time', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('minimal_availability_percent', 'float', [
                    'default'   => null,
                    'null'      => true,
                    'precision' => 8,
                    'scale'     => 3,
                ])
                ->addColumn('determined_availability_percent', 'float', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addColumn('maximal_number_outages', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => true,
                ])
                ->addColumn('determined_number_outages', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => true,
                ])
                ->addColumn('created', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addIndex(
                    [
                        'autoreport_id',
                        'last_update',
                    ]
                )
                ->create();
        }

        if (!$this->hasTable('hosts_to_autoreports')) {
            $this->table('hosts_to_autoreports')
                ->addColumn('id', 'integer', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 11,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('autoreport_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('host_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('outage_duration', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => true,
                ])
                ->addColumn('configuration_option', 'integer', [
                    'default' => '0',
                    'limit'   => 4,
                    'null'    => false,
                ])
                ->addColumn('created', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addColumn('modified', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addIndex(
                    [
                        'autoreport_id',
                        'host_id',
                    ],
                    ['unique' => true]
                )
                ->addIndex(
                    [
                        'host_id',
                    ]
                )
                ->create();
        }

        if (!$this->hasTable('services_to_autoreports')) {
            $this->table('services_to_autoreports')
                ->addColumn('id', 'integer', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 11,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('autoreport_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('host_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('service_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('outage_duration', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => true,
                ])
                ->addColumn('configuration_option', 'integer', [
                    'default' => '0',
                    'limit'   => 4,
                    'null'    => false,
                ])
                ->addColumn('graph_settings', 'string', [
                    'default' => null,
                    'limit'   => 10,
                    'null'    => false,
                ])
                ->addColumn('created', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addColumn('modified', 'datetime', [
                    'default' => null,
                    'limit'   => null,
                    'null'    => false,
                ])
                ->addIndex(
                    [
                        'autoreport_id',
                        'service_id',
                    ],
                    ['unique' => true]
                )
                ->addIndex(
                    [
                        'service_id',
                    ]
                )
                ->create();
        }

        if (!$this->hasTable('users_to_autoreports')) {
            $this->table('users_to_autoreports')
                ->addColumn('id', 'integer', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 11,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('autoreport_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addColumn('user_id', 'integer', [
                    'default' => null,
                    'limit'   => 11,
                    'null'    => false,
                ])
                ->addIndex(
                    [
                        'user_id',
                    ]
                )
                ->create();
        }
    }

    /**
     * Down Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-down-method
     * @return void
     */
    public function down(): void {
        $this->table('autoreport_quarterlies')->drop()->save();
        $this->table('autoreport_settings')->drop()->save();
        $this->table('autoreports')->drop()->save();
        $this->table('autoreports_availability_log')->drop()->save();
        $this->table('hosts_to_autoreports')->drop()->save();
        $this->table('services_to_autoreports')->drop()->save();
        $this->table('users_to_autoreports')->drop()->save();
    }
}
