<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace AutoreportModule\Command;


use AutoreportModule\Model\Table\AutoreportAvailabilityLogTable;
use AutoreportModule\Model\Table\AutoreportsTable;
use Cake\Console\Arguments;
use Cake\Console\Command;
use Cake\Console\ConsoleIo;
use Cake\Console\ConsoleOptionParser;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Core\Interfaces\CronjobInterface;

/**
 * Class AutoreportCalculateAvailabilityCommand
 * @package AutoreportModule\Command
 */
class AutoreportCalculateAvailabilityCommand extends Command implements CronjobInterface {

    /**
     * @var ConsoleIo
     */
    private $io;

    /**
     * Hook method for defining this command's option parser.
     *
     * @see https://book.cakephp.org/3.0/en/console-and-shells/commands.html#defining-arguments-and-options
     *
     * @param \Cake\Console\ConsoleOptionParser $parser The parser to be defined
     * @return \Cake\Console\ConsoleOptionParser The built parser.
     */
    public function buildOptionParser(ConsoleOptionParser $parser): ConsoleOptionParser {
        $parser = parent::buildOptionParser($parser);

        return $parser;
    }

    /**
     * @param Arguments $args
     * @param ConsoleIo $io
     * @return int|void|null
     * @throws \App\Lib\Exceptions\MissingDbBackendException
     * @throws \GuzzleHttp\Exception\GuzzleException
     * @throws \Exception
     */
    public function execute(Arguments $args, ConsoleIo $io) {
        $this->io = $io;

        /** @var $AutoreportsTable AutoreportsTable */
        $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');

        /** @var $AutoreportAvailabilityLogTable AutoreportAvailabilityLogTable */
        $AutoreportAvailabilityLogTable = TableRegistry::getTableLocator()->get('AutoreportModule.AutoreportAvailabilityLog');

        $io->out('Calculate autoreport availability ...');

        $autoreportsForCalculateAvailability = $AutoreportsTable->getAutoreportsAsListForCalculateAvailabilityCronjob();

        foreach ($autoreportsForCalculateAvailability as $autoreport) {
            $hasToBeCalculated = true;
            $lastReportLogEntry = $AutoreportAvailabilityLogTable->getLastAutoreportAvailabilityLogByAutoreportId($autoreport['id']);
            if (!empty($lastReportLogEntry)) {
                $lastUpdateDate = new \DateTime(date('d.m.Y H:i:s', $lastReportLogEntry['last_update']));
                switch ($autoreport['report_interval']) {
                    case 'DAY':
                        $lastUpdateDate->modify('+1 hour');
                        if ($lastUpdateDate->getTimestamp() > time()) {
                            $hasToBeCalculated = false;
                        }
                        break;
                    default:
                        $lastUpdateDate->modify('+1 day');
                        if ($lastUpdateDate->getTimestamp() > time()) {
                            $hasToBeCalculated = false;
                        }
                        break;
                }
                if (!$hasToBeCalculated) {
                    continue;
                }
            }

            $calculatedRanges = $AutoreportAvailabilityLogTable->calculateAvailabilityForReport($autoreport['id']);

            foreach ($calculatedRanges as $dateRange) {
                $io->out('"' . $autoreport['name'] . '" ' . date('d.m.Y H:i:s', $dateRange['start']) . ' - ' . date('d.m.Y H:i:s', $dateRange['end']) . '<green>' . " \xE2\x9C\x93" . '</green>');
            }
            $io->hr();
        }
        $io->success('Done');
    }
}
