<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace AutoreportModule\Controller;

use App\Model\Table\ContainersTable;
use App\Model\Table\HostsTable;
use App\Model\Table\ServicesTable;
use App\Model\Table\WidgetsTable;
use AutoreportModule\Filter\AutoreportFilter;
use AutoreportModule\Form\AutoreportForm;
use AutoreportModule\Lib\Autoreport;
use AutoreportModule\Lib\AutoreportServiceConditions;
use AutoreportModule\Lib\CsvReport;
use AutoreportModule\Model\Table\AutoreportAvailabilityLogTable;
use AutoreportModule\Model\Table\AutoreportSettingsTable;
use AutoreportModule\Model\Table\AutoreportsHostsMembershipsTable;
use AutoreportModule\Model\Table\AutoreportsServicesMembershipsTable;
use AutoreportModule\Model\Table\AutoreportsTable;
use Cake\Http\Exception\MethodNotAllowedException;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\TableRegistry;
use Cake\Utility\Hash;
use itnovum\openITCOCKPIT\Core\AngularJS\Api;
use itnovum\openITCOCKPIT\Core\ValueObjects\User;
use itnovum\openITCOCKPIT\Core\Views\UserTime;
use itnovum\openITCOCKPIT\Database\PaginateOMat;

/**
 * Class AutoreportsController
 * @package AutoreportModule\Controller
 */
class AutoreportsController extends AppController {

    public function index() {
        if (!$this->isApiRequest()) {
            //Only ship template for AngularJs
            return;
        }
        /** @var AutoreportFilter $AutoreportFilter */
        $AutoreportFilter = new AutoreportFilter($this->request);
        /** @var $AutoreportsTable AutoreportsTable */
        $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');

        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $AutoreportFilter->getPage());
        $MY_RIGHTS = [];
        if ($this->hasRootPrivileges === false) {
            /** @var $ContainersTable ContainersTable */
            //$ContainersTable = TableRegistry::getTableLocator()->get('Containers');
            //$MY_RIGHTS = $ContainersTable->resolveChildrenOfContainerIds($this->MY_RIGHTS);
            // ITC-2863 $this->MY_RIGHTS is already resolved and contains all containerIds a user has access to
            $MY_RIGHTS = $this->MY_RIGHTS;
        }
        $autoreports = $AutoreportsTable->getAutoreportsIndex($AutoreportFilter, $PaginateOMat, $MY_RIGHTS);
        foreach ($autoreports as $index => $autoreport) {
            $autoreports[$index]['allowEdit'] = $this->isWritableContainer($autoreport['container_id']);
        }

        $this->set('autoreports', $autoreports);
        $this->viewBuilder()->setOption('serialize', ['autoreports']);
    }

    //Only for ACLs
    public function add() {
        return;
    }

    public function addStepOne() {
        if (!$this->isApiRequest()) {
            //Only ship template for AngularJs
            return;
        }

        /** @var $AutoreportsTable AutoreportsTable */
        $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');
        if ($this->request->is('post') && $this->isAngularJsRequest()) {

            $request = $this->request->getData('Autoreport', []);
            if (isset($request['use_start_time']) && $request['use_start_time'] && $request['report_start_date']) {
                $date = \DateTime::createFromFormat('d.m.Y', $request['report_start_date']);
                $request['report_start_date'] = $date->format('Y-m-d');
            }
            if ($request['min_availability_percent'] === true) {
                //cast for validate with 'min_availability' regex
                $request['min_availability'] = (string)$request['min_availability'];
            }

            $autoreport = $AutoreportsTable->newEmptyEntity();
            $autoreport = $AutoreportsTable->patchEntity($autoreport, $request, [
                'validate' => 'addStepOne' //function call AutoreportsTable::validationAddStepOne
            ]);
            $AutoreportsTable->save($autoreport);
            if ($autoreport->hasErrors()) {
                $this->response = $this->response->withStatus(400);
                $this->serializeCake4ErrorMessage($autoreport);
                return;
            }
            $this->set('autoreport', $autoreport);
            $this->viewBuilder()->setOption('serialize', ['autoreport']);
        }
    }


    /**
     * @param int|null $id
     */
    public function addStepTwo($id = null) {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        /** @var AutoreportsTable $AutoreportsTable */
        $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');

        if (!$AutoreportsTable->existsById($id)) {
            throw new NotFoundException(__('Auto report not found'));
        }

        $autoreport = $AutoreportsTable->getAutoreportById($id);

        if ($this->request->is('get') && $this->isAngularJsRequest()) {
            //Return auto report information
            $this->set('autoreport', $autoreport);
            $this->viewBuilder()->setOption('serialize', ['autoreport']);
            return;
        }

        if ($this->request->is('post') || $this->request->is('put')) {
            $autoreport = $AutoreportsTable->get($id);
            $autoreport = $AutoreportsTable->patchEntity($autoreport, $this->request->getData('Autoreport', []), [
                'validate' => 'addStepTwo' //function call AutoreportsTable::validationAddStepTwo
            ]);


            $data = $AutoreportsTable->prepareStepTwoDataForSave($id, $this->request->getData('Autoreport', []));
            $entity = $AutoreportsTable->get($id);


            $entity = $AutoreportsTable->patchEntity($entity, $data, [
                'validate' => false
            ]);

            $AutoreportsTable->save($entity);
            if ($entity->hasErrors()) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', $entity->getErrors());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }

            $newHostIds = Hash::extract($data['hosts'], '{n}.id');
            $newServiceIds = Hash::extract($data['services'], '{n}.id');

            $AutoreportsTable->saveUsageFlags(
                $id,
                $newHostIds,
                [],
                $newServiceIds,
                []
            );

            $this->set('autoreport', $entity);
            $this->viewBuilder()->setOption('serialize', ['autoreport']);
        }
    }

    //Only for ACLs
    public function edit() {
        return;
    }

    /**
     * @param int|null $id
     */
    public function editStepOne($id = null) {
        if (!$this->isApiRequest()) {
            //Only ship template for AngularJs
            return;
        }

        /** @var AutoreportsTable $AutoreportsTable */
        $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');

        if (!$AutoreportsTable->existsById($id)) {
            throw new NotFoundException(__('Auto report not found'));
        }

        $autoreport = $AutoreportsTable->get($id);

        if (!$this->allowedByContainerId($autoreport->get('container_id'), false)) {
            $this->render403();
            return;
        }

        if ($this->request->is('get')) {
            $autoreport = $AutoreportsTable->getAutoreportForEditStepOne($id);
            if ($autoreport['use_start_time']) {
                $autoreport['report_start_date'] = $autoreport['report_start_date']->format('d.m.Y');
            }


            $this->set('autoreport', $autoreport);
            $this->viewBuilder()->setOption('serialize', ['autoreport']);
            return;
        }


        if ($this->request->is('post') && $this->isAngularJsRequest()) {
            $request = $this->request->getData('Autoreport', []);
            if (isset($request['use_start_time']) && $request['use_start_time'] && $request['report_start_date']) {
                $date = \DateTime::createFromFormat('d.m.Y', $request['report_start_date']);
                $request['report_start_date'] = $date->format('Y-m-d');
            }
            if ($request['min_availability_percent'] === true) {
                //cast for validate with 'min_availability' regex
                $request['min_availability'] = (string)$request['min_availability'];
            }

            $autoreport = $AutoreportsTable->patchEntity($autoreport, $request, [
                'validate' => 'addStepOne' //function call AutoreportsTable::validationAddStepOne
            ]);

            $AutoreportsTable->save($autoreport);
            if ($autoreport->hasErrors()) {
                $this->response = $this->response->withStatus(400);
                $this->serializeCake4ErrorMessage($autoreport);
                return;
            }
            $this->set('autoreport', $autoreport);
            $this->viewBuilder()->setOption('serialize', ['autoreport']);
        }
    }

    /**
     * @param int|null $id
     */
    public function editStepTwo($id = null) {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        /** @var AutoreportsTable $AutoreportsTable */
        $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');

        if (!$AutoreportsTable->existsById($id)) {
            throw new NotFoundException(__('Auto report not found'));
        }

        $autoreport = $AutoreportsTable->getAutoreportForEditStepTwo($id);
        $selectedHostIds = Hash::merge(
            Hash::extract($autoreport['hosts'], '{n}.id'),
            Hash::extract($autoreport['services'], '{n}.host_id')
        );
        $selectedHostIds = array_unique($selectedHostIds);
        $selectedHostIds = array_values($selectedHostIds); //Make it an array [] not object {}

        $oldHostIds = $selectedHostIds;
        $oldServiceIds = Hash::extract($autoreport['services'], '{n}.id');

        if (!$this->allowedByContainerId($autoreport['container_id'], false)) {
            $this->render403();
            return;
        }

        if ($this->request->is('get')) {
            //Get used host Ids for the select box


            //Return auto report information
            $this->set('autoreport', $autoreport);
            $this->set('selectedHostIds', $selectedHostIds);
            $this->viewBuilder()->setOption('serialize', ['autoreport', 'selectedHostIds']);
            return;
        }

        if ($this->request->is('post') || $this->request->is('put')) {
            $autoreport = $AutoreportsTable->get($id);
            $autoreport = $AutoreportsTable->patchEntity($autoreport, $this->request->getData('Autoreport', []), [
                'validate' => 'addStepTwo' //function call AutoreportsTable::validationAddStepTwo
            ]);


            $data = $AutoreportsTable->prepareStepTwoDataForSave($id, $this->request->getData('Autoreport', []));
            $entity = $AutoreportsTable->get($id);


            $entity = $AutoreportsTable->patchEntity($entity, $data, [
                'validate' => false
            ]);

            $AutoreportsTable->save($entity);
            if ($entity->hasErrors()) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', $entity->getErrors());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }

            $newHostIds = Hash::extract($data['hosts'], '{n}.id');
            $newServiceIds = Hash::extract($data['services'], '{n}.id');

            $AutoreportsTable->saveUsageFlags(
                $id,
                $newHostIds,
                array_diff($oldHostIds, $newHostIds),
                $newServiceIds,
                array_diff($oldServiceIds, $newServiceIds)
            );

            $this->set('autoreport', $entity);
            $this->viewBuilder()->setOption('serialize', ['autoreport']);
        }
    }

    /**
     * @param int|null $id
     */
    public function editStepThree($id = null) {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        /** @var AutoreportsTable $AutoreportsTable */
        $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');

        if (!$AutoreportsTable->existsById($id)) {
            throw new NotFoundException(__('Auto report not found'));
        }

        $autoreport = $AutoreportsTable->getAutoreportForOverview($id);

        if (!$this->allowedByContainerId($autoreport['container_id'], false)) {
            $this->render403();
            return;
        }

        $this->set('autoreport', $autoreport);
        $this->viewBuilder()->setOption('serialize', ['autoreport']);
    }


    /**
     * @throws \App\Lib\Exceptions\MissingDbBackendException
     * @throws \GuzzleHttp\Exception\GuzzleException
     * @throws \Exception
     */
    public function generate() {

        if ($this->isHtmlRequest()) {
            //Only ship HTML template
            return;
        }
        /** @var AutoreportSettingsTable $AutoreportSettingsTable */
        $AutoreportSettingsTable = TableRegistry::getTableLocator()->get('AutoreportModule.AutoreportSettings');
        $settings = $AutoreportSettingsTable->currentAutoreportSettings();
        $defaultSettings = $AutoreportSettingsTable->defaultAutoreportSettings();
        $User = new \itnovum\openITCOCKPIT\Core\ValueObjects\User($this->getUser());
        $UserTime = $User->getUserTime();
        //User server timezone but with user date format
        $ServerTime = new UserTime(date_default_timezone_get(), $UserTime->getFormatString());

        /** @var $AutoreportsTable AutoreportsTable */
        $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');
        if ($this->request->is('get') && (!$this->isPdfRequest() || !$this->isZipRequest())) {
            $MY_RIGHTS = [];
            if ($this->hasRootPrivileges === false) {
                /** @var $ContainersTable ContainersTable */
                //$ContainersTable = TableRegistry::getTableLocator()->get('Containers');
                //$MY_RIGHTS = $ContainersTable->resolveChildrenOfContainerIds($this->MY_RIGHTS);
                // ITC-2863 $this->MY_RIGHTS is already resolved and contains all containerIds a user has access to
                $MY_RIGHTS = $this->MY_RIGHTS;
            }
            $AutoreportFilter = new AutoreportFilter($this->request);
            $autoreports = $AutoreportsTable->getAutoreportsIndex($AutoreportFilter, null, $MY_RIGHTS);
            foreach ($autoreports as $index => $autoreport) {
                $autoreports[$index]['allowEdit'] = $this->isWritableContainer($autoreport['container_id']);
            }

            $this->set('autoreports', $autoreports);
            $this->viewBuilder()->setOption('serialize', ['autoreports']);
        }

        if ($this->request->is('post')) {
            $AutoreportForm = new AutoreportForm();
            $AutoreportForm->execute($this->request->getData('Autoreport', []));

            if (!empty($AutoreportForm->getErrors())) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', $AutoreportForm->getErrors());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }


            $Autoreport = new Autoreport(
                $this->request->getData('Autoreport.id', -1),
                strtotime($this->request->getData('Autoreport.from_date', date('d.m.Y')) . ' 00:00:00'),
                strtotime($this->request->getData('Autoreport.to_date', date('d.m.Y')) . ' 23:59:59'),
                $ServerTime
            );

            $reportFormat = $this->request->getData('Autoreport.format', 'html');
            $result = $Autoreport->canReportBeCreated();
            if ($result === true) {
                if ($reportFormat !== 'html' && $reportFormat !== 'json') {
                    //Report validated successfully.
                    //PDF and CSV reports will get generated by an GET request.

                    $autoreport = $AutoreportsTable->get($this->request->getData('Autoreport.id', -1));
                    $filename = $autoreport->get('name');
                    $filenameSuffix = '';
                    if ($settings['configuration_option'] & $defaultSettings['AUTOREPORTS_USE_CREATE_DATE_IN_FILENAME']['value']) {
                        $filenameSuffix .= '_' . date('dmY', time());
                    }
                    if ($settings['configuration_option'] & $defaultSettings['AUTOREPORTS_USE_YEAR_IN_FILENAME']['value']) {
                        if (strlen($filenameSuffix) === 0) {
                            $filenameSuffix .= '_' . date('Y', strtotime($this->request->getQuery('from_date', date('d.m.Y')) . ' 00:00:00'));
                        }
                    }
                    $this->set('success', true);
                    $this->set('filename', $filename . $filenameSuffix . '.' . $reportFormat);
                    $this->viewBuilder()->setOption('serialize', ['success', 'filename']);
                    return;
                }

                //Create report HTML / json report
                $report = $Autoreport->generateReport(true);

                //remove data we don't need anymore
                unset($report['autoreport_data']['time_slices']);

                $this->set('success', true);
                $this->set('report', $report);
                $this->viewBuilder()->setOption('serialize', [
                    'success',
                    'report'
                ]);
            }

            if (!empty($result['error'])) {
                $this->response = $this->response->withStatus(400);
                $this->set('report_error', $result);
                $this->viewBuilder()->setOption('serialize', ['report_error']);
                return;
            }
        }
        if ($this->request->is('get') && $this->isPdfRequest()) {
            //Generate PDF report

            $startTimestamp = strtotime($this->request->getQuery('data.from_date', date('d.m.Y')) . ' 00:00:00');
            $endTimestamp = strtotime($this->request->getQuery('data.to_date', date('d.m.Y')) . ' 23:59:59');

            $User = new \itnovum\openITCOCKPIT\Core\ValueObjects\User($this->getUser());
            $UserTime = $User->getUserTime();
            $Autoreport = new Autoreport(
                $this->request->getQuery('data.id', -1),
                $startTimestamp,
                $endTimestamp,
                $UserTime
            );

            $result = $Autoreport->canReportBeCreated();
            if ($result !== true) {
                //Send a POST request first to validate your DATA!!!
                throw new \Exception($result['message']);
            }

            //Create report PDF
            $report = $Autoreport->generateReport(true);

            //remove data we don't need anymore
            unset($report['autoreport_data']['time_slices']);

            $filename = $report['autoreport']['name'];
            $filenameSuffix = '';
            if ($settings['configuration_option'] & $defaultSettings['AUTOREPORTS_USE_CREATE_DATE_IN_FILENAME']['value']) {
                $filenameSuffix .= '_' . date('dmY', time());
            }
            if ($settings['configuration_option'] & $defaultSettings['AUTOREPORTS_USE_YEAR_IN_FILENAME']['value']) {
                if (strlen($filenameSuffix) === 0) {
                    $filenameSuffix .= '_' . date('Y', strtotime($this->request->getQuery('from_date', date('d.m.Y')) . ' 00:00:00'));
                }
            }

            $this->set('success', true);
            $this->set('report', $report);
            $this->set('UserTime', $ServerTime);
            $this->set('reportStart', $ServerTime->format($startTimestamp));
            $this->set('reportEnd', $ServerTime->format($endTimestamp));

            $this->viewBuilder()->setOption(
                'pdfConfig',
                [
                    'download' => true,
                    'filename' => $filename . $filenameSuffix . '.pdf'
                ]
            );
        }
        if ($this->request->is('get') && $this->isZipRequest()) {
            //Generate report - ZIP with CSV files
            $User = new \itnovum\openITCOCKPIT\Core\ValueObjects\User($this->getUser());
            $UserTime = $User->getUserTime();
            $Autoreport = new Autoreport(
                $this->request->getQuery('data.id', -1),
                strtotime($this->request->getQuery('data.from_date', date('d.m.Y')) . ' 00:00:00'),
                strtotime($this->request->getQuery('data.to_date', date('d.m.Y')) . ' 23:59:59'),
                $ServerTime
            );

            $result = $Autoreport->canReportBeCreated();
            if ($result !== true) {
                //Send a POST request first to validate your DATA!!!
                throw new \Exception($result['message']);
            }

            $reportStart = strtotime($this->request->getQuery('data.from_date', date('d.m.Y')) . ' 00:00:00');

            //Create report CSV
            $report = $Autoreport->generateReport(false);
            //remove data we don't need anymore
            unset($report['autoreport_data']['time_slices']);
            $path = !empty($settings['store_path']) ? $settings['store_path'] . DS : ROOT . DS . 'plugins/AutoreportModule/webroot/tmp/';

            if (!is_dir($path)) {
                mkdir($path, 0755, true);
            }

            $CsvReport = new CsvReport($report, $reportStart, $path, $settings, $defaultSettings);
            $zipName = $CsvReport->getReportAsZip();

            $this->disableAutoRender();
            $this->set('success', true);
            $this->set('report', $report);
            $this->set('UserTime', $UserTime);

            $this->response = $this->response->withFile(
                $zipName,
                [
                    'download' => true,
                ]
            );
        }
    }


    /**
     * @param int|null $id
     */
    public function delete($id = null) {
        if (!$this->request->is('post')) {
            throw new MethodNotAllowedException();
        }

        /** @var AutoreportsTable $AutoreportsTable */
        $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');

        if (!$AutoreportsTable->existsById($id)) {
            throw new NotFoundException(__('Auto report not found'));
        }

        $autoreport = $AutoreportsTable->getAutoreportForDelete($id);

        if (!$this->allowedByContainerId($autoreport->get('container_id'))) {
            $this->render403();
            return;
        }

        $hostIds = Hash::extract($autoreport->toArray(), 'hosts.{n}.id');
        $serviceIds = Hash::extract($autoreport->toArray(), 'services.{n}.id');

        if ($AutoreportsTable->delete($autoreport)) {
            //Remove usage flags
            $AutoreportsTable->saveUsageFlags(
                $id,
                [],
                $hostIds,
                [],
                $serviceIds
            );


            $this->set('success', true);
            $this->viewBuilder()->setOption('serialize', ['success']);

            return;
        }

        $this->response = $this->response->withStatus(400);
        $this->set('success', false);
        $this->viewBuilder()->setOption('serialize', ['success']);
        return;
    }


    /**
     * @param int|null $id
     */
    public function hostUsedBy($hostId = null) {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');

        if (!$HostsTable->existsById($hostId)) {
            throw new NotFoundException(__('Host not found'));
        }

        $host = $HostsTable->getHostByIdForPermissionCheck($hostId);

        if (!$this->allowedByContainerId($host->getContainerIds(), false)) {
            $this->render403();
            return;
        }

        /** @var AutoreportsTable $AutoreportsTable */
        $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');
        /** @var AutoreportsHostsMembershipsTable $AutoreportsHostsMembershipsTable */
        $AutoreportsHostsMembershipsTable = TableRegistry::getTableLocator()->get('AutoreportModule.AutoreportsHostsMemberships');
        /** @var AutoreportsServicesMembershipsTable $AutoreportsServicesMembershipsTable */
        $AutoreportsServicesMembershipsTable = TableRegistry::getTableLocator()->get('AutoreportModule.AutoreportsServicesMemberships');

        //Get all host ids where the host itself is in an auto report, or an service of the host is in the report
        $autoreportIds = array_merge(
            $AutoreportsHostsMembershipsTable->getAllAutoreportIdsByHostId($hostId),
            $AutoreportsServicesMembershipsTable->getAllAutoreportIdsByHostId($hostId)
        );

        $autoreports = [];
        if (!empty($autoreportIds)) {
            $autoreports = $AutoreportsTable->getAutoreportsByIds($autoreportIds, $this->MY_RIGHTS);

            foreach ($autoreports as $index => $autoreport) {
                $autoreports[$index]['allowEdit'] = $this->isWritableContainer($autoreport['container_id']);
            }
        }

        $this->set('host', $host);
        $this->set('autoreports', $autoreports);
        $this->viewBuilder()->setOption('serialize', ['host', 'autoreports']);
    }

    /**
     * @param int|null $serviceId
     */
    public function serviceUsedBy($serviceId = null) {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        /** @var ServicesTable $ServicesTable */
        $ServicesTable = TableRegistry::getTableLocator()->get('Services');

        if (!$ServicesTable->existsById($serviceId)) {
            throw new NotFoundException(__('Service not found'));
        }

        $service = $ServicesTable->getServiceByIdForPermissionsCheck($serviceId);
        if (!$this->allowedByContainerId($service->getContainerIds(), false)) {
            $this->render403();
            return;
        }

        /** @var AutoreportsTable $AutoreportsTable */
        $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');
        /** @var AutoreportsHostsMembershipsTable $AutoreportsHostsMembershipsTable */
        $AutoreportsHostsMembershipsTable = TableRegistry::getTableLocator()->get('AutoreportModule.AutoreportsHostsMemberships');
        /** @var AutoreportsServicesMembershipsTable $AutoreportsServicesMembershipsTable */
        $AutoreportsServicesMembershipsTable = TableRegistry::getTableLocator()->get('AutoreportModule.AutoreportsServicesMemberships');

        //Get all host ids where the host itself is in an auto report, or an service of the host is in the report
        $autoreportIds = $AutoreportsServicesMembershipsTable->getAllAutoreportIdsByServiceId($serviceId);

        $autoreports = [];
        if (!empty($autoreportIds)) {
            $autoreports = $AutoreportsTable->getAutoreportsByIds($autoreportIds, $this->MY_RIGHTS);

            foreach ($autoreports as $index => $autoreport) {
                $autoreports[$index]['allowEdit'] = $this->isWritableContainer($autoreport['container_id']);
            }
        }

        $this->set('service', $service);
        $this->set('autoreports', $autoreports);
        $this->viewBuilder()->setOption('serialize', ['service', 'autoreports']);
    }

    /**
     * @throws \Exception
     * @deprecated
     */
    public function autoreportWidget() {
        if (!$this->isApiRequest()) {
            //Only ship HTML template
            return;
        }
        $User = new User($this->getUser());
        $UserTime = UserTime::fromUser($User);
        $config = [
            'Autoreport' => [
                'id'                    => 0,
                'in_percent'            => true,
                'set_color_dynamically' => false,
                'show_update_date'      => true
            ],
        ];

        /** @var WidgetsTable $WidgetsTable */
        $WidgetsTable = TableRegistry::getTableLocator()->get('Widgets');

        if ($this->request->is('get')) {
            $widgetId = (int)$this->request->getQuery('widgetId', 0);
            if (!$WidgetsTable->existsById($widgetId)) {
                throw new \RuntimeException('Invalid widget id');
            }
            $widget = $WidgetsTable->getWidgetByIdAsCake2($widgetId);

            if ($widget['Widget']['json_data'] !== null && $widget['Widget']['json_data'] !== '') {
                $json = @json_decode($widget['Widget']['json_data'], true);
                $config = Hash::merge($config, $json);
            }

            /** @var AutoreportsTable $AutoreportsTable */
            $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');
            $autoreport = $AutoreportsTable->getAutoreportByIdForAvailabilityWidget($config['Autoreport']['id'], false);
            if (!empty($autoreport)) {
                if (!empty($autoreport['autoreport_availability_log'])) {
                    $autoreport['autoreport_availability_log'] = Hash::sort(
                        $autoreport['autoreport_availability_log'],
                        '{n}.id',
                        'asc'
                    );
                    $availableLogCounter = sizeof($autoreport['autoreport_availability_log']);
                    foreach ($autoreport['autoreport_availability_log'] as $key => $availabilityLog) {
                        $autoreport['autoreport_availability_log'][$key]['determined_availability_time_human'] = $UserTime->secondsInHumanShort(
                            $autoreport['autoreport_availability_log'][$key]['determined_availability_time']
                        );
                        $autoreport['autoreport_availability_log'][$key]['last_update_human'] = $UserTime->format(
                            $autoreport['autoreport_availability_log'][$key]['last_update']
                        );
                        //last record
                        if ($key === ($availableLogCounter - 1)) {
                            $autoreport['min_availability'] = $availabilityLog['minimal_availability_percent'];
                            $autoreport['last_absolut_value'] = $availabilityLog['determined_availability_time'];
                            $autoreport['last_percent_value'] = $availabilityLog['determined_availability_percent'];
                            $autoreport['last_send_date'] = $availabilityLog['last_update'];
                        }
                    }
                }

                $autoreport['last_absolut_value_human'] = $UserTime->secondsInHumanShort($autoreport['last_absolut_value']);
                $autoreport['last_send_date'] = !is_null($autoreport['last_send_date']) ? $UserTime->format($autoreport['last_send_date']) : null;
            } else {
                $autoreport['id'] = null;
            }
            $acl = [
                'autoreport' => [
                    'generate' => isset($this->PERMISSIONS['autoreportmodule']['autoreports']['generate']),
                ]
            ];


            $this->set('autoreport', $autoreport);
            $this->set('config', $config);
            $this->set('ACL', $acl);

            $this->viewBuilder()->setOption('serialize', [
                'autoreport',
                'config',
                'ACL'
            ]);
            return;
        }

        if ($this->request->is('post')) {
            $config['Autoreport'] = $this->request->getData('Autoreport', []);
            $widgetId = (int)$this->request->getData('Widget.id', 0);

            /** @var WidgetsTable $WidgetsTable */
            if (!$WidgetsTable->existsById($widgetId)) {
                throw new \RuntimeException('Invalid widget id');
            }

            $widget = $WidgetsTable->get($widgetId);
            $widget->set('json_data', json_encode($config));

            $WidgetsTable->save($widget);
            if ($widget->hasErrors()) {
                $this->serializeCake4ErrorMessage($widget);
                return;
            } else {
                $this->set('config', $config);
                $this->viewBuilder()->setOption('serialize', ['config']);
                return;
            }


        }
        throw new MethodNotAllowedException();
    }

    /**
     * @param null $id
     */
    public function availabilityView($id = null) {
        if ($this->isHtmlRequest()) {
            //Only ship HTML template
            return;
        }
        $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');
        if (!$AutoreportsTable->existsByIdForAvailabilityCalculation($id)) {
            throw new NotFoundException(__('Auto report not found'));
        }
        /** @var $AutoreportAvailabilityLogTable AutoreportAvailabilityLogTable */
        $AutoreportAvailabilityLogTable = TableRegistry::getTableLocator()->get('AutoreportModule.AutoreportAvailabilityLog');

        try {
            $autoreportAvailabilityLog = $AutoreportAvailabilityLogTable->calculateAvailabilityForReport($id);
            if ($this->isJsonRequest()) {
                $this->set('calculatedTimeRanges', $autoreportAvailabilityLog);
                $this->viewBuilder()->setOption('serialize', ['calculatedTimeRanges']);
            }
        } catch (MissingDbBackendException $e) {
        } catch (GuzzleException $e) {
        }

    }

    public function autoreportAvailibilityColors() {
        $this->layout = 'blank';
        //Only ship HTML Template
        return;
    }

    /**
     * @param int $id
     * @throws \App\Lib\Exceptions\MissingDbBackendException
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function calculateAvailability($id) {
        /** @var $AutoreportsTable AutoreportsTable */
        $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');
        if (!$AutoreportsTable->existsByIdForAvailabilityCalculation($id)) {
            throw new NotFoundException(__('Auto report not found'));
        }

        /** @var $AutoreportAvailabilityLogTable AutoreportAvailabilityLogTable */
        $AutoreportAvailabilityLogTable = TableRegistry::getTableLocator()->get('AutoreportModule.AutoreportAvailabilityLog');

        $autoreportAvailabilityLog = $AutoreportAvailabilityLogTable->calculateAvailabilityForReport($id);

        if (isset($autoreportAvailabilityLog['hasError'])) {
            $this->serializeCake4ErrorMessage($autoreportAvailabilityLog['data']);
            return;
        }

        if ($this->isJsonRequest()) {
            $this->set('calculatedTimeRanges', $autoreportAvailabilityLog);
            $this->viewBuilder()->setOption('serialize', ['calculatedTimeRanges']);
        }

    }

    /****************************
     *       AJAX METHODS       *
     ****************************/

    public function hostAndServicesByHostId() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        $hostIds = $this->request->getQuery('hostIds');
        $serviceNameRegex = $this->request->getQuery('servicenameRegex');
        $selected = $this->request->getQuery('selected');

        /** @var $AutoreportServiceConditions AutoreportServiceConditions */
        $AutoreportServiceConditions = new AutoreportServiceConditions();
        $AutoreportServiceConditions->setHostIds($hostIds);
        $AutoreportServiceConditions->setServicenameRegex($serviceNameRegex);


        /** @var $AutoreportsTable AutoreportsTable */
        $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');
        $hostsWithServices = $AutoreportsTable->getAutoreportServicesForAngularCake4($AutoreportServiceConditions, $this->MY_RIGHTS, $selected);
        $serviceIdsList = Hash::extract($hostsWithServices, '{n}.services.{n}.id');

        $this->set('hostsWithServices', $hostsWithServices);
        $this->set('serviceIdsList', $serviceIdsList);
        $this->viewBuilder()->setOption('serialize', ['hostsWithServices', 'serviceIdsList']);
    }

    public function loadAutoreports() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        $availabilityLog = $this->request->getQuery('availabilityLog', false) === 'true';
        /** @var $AutoreportsTable AutoreportsTable */
        $AutoreportsTable = TableRegistry::getTableLocator()->get('AutoreportModule.Autoreports');
        $autoreports = $AutoreportsTable->getAutoreportsByContainerIdsAsList(
            $this->MY_RIGHTS,
            $availabilityLog
        );
        $autoreports = Api::makeItJavaScriptAble(
            $autoreports
        );

        $this->set('autoreports', $autoreports);
        $this->viewBuilder()->setOption('serialize', ['autoreports']);
    }
}
