<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


namespace AutoreportModule\Lib;


use itnovum\openITCOCKPIT\Core\Reports\CsvCreator;

/**
 * Class CsvReport
 * @package AutoreportModule\Lib
 */
class CsvReport {

    /**
     * @var array
     */
    private $report;

    /**
     * @var int timestamp
     */
    private $reportStart;

    /** @var string */
    private $path;

    /**
     * @var array
     */
    private $settings;

    /**
     * @var array
     */
    private $defaultSettings;

    /**
     * CsvReport constructor.
     * @param $this ->report
     * @param $this ->reportStart
     * @param $path
     * @param $settings
     * @param $defaultSettings
     */
    public function __construct($report, $reportStart, $path, $settings, $defaultSettings) {
        $this->report = $report;
        $this->reportStart = $reportStart;
        $this->path = $path;
        $this->settings = $settings;
        $this->defaultSettings = $defaultSettings;
    }

    public function getReportAsZip() {

        $filename = $this->report['autoreport']['name'];
        $filenameSuffix = '';
        if ($this->settings['configuration_option'] & $this->defaultSettings['AUTOREPORTS_USE_CREATE_DATE_IN_FILENAME']['value']) {
            $filenameSuffix .= '_' . date('dmY', time());
        }
        if ($this->settings['configuration_option'] & $this->defaultSettings['AUTOREPORTS_USE_YEAR_IN_FILENAME']['value']) {
            if (strlen($filenameSuffix) === 0) {
                $filenameSuffix .= '_' . date('Y', $this->reportStart);
            }
        }
        $showOutagesInDowntimes = (
            $this->settings['configuration_option'] &
            $this->defaultSettings['AUTOREPORTS_SHOW_OUTAGES_IN_DOWNTIME']['value']
        );

        $filename = $filename . $filenameSuffix . '.zip';

        $zipName = $this->path . $filename;
        $baseZipName = str_replace('.zip', '', $filename);

        $ParameterCsv = $this->path . $baseZipName . '_Parameter_' . date('Ymd_His') . '.csv';
        $DetailsCSV = $this->path . $baseZipName . '_Details_' . date('Ymd_His') . '.csv';

        $timeRangeText = '';
        if (!empty($this->report['autoreport']['timeperiod']['timeperiod_timeranges'])) {
            $timeRangeArr = [];
            foreach ($this->report['autoreport']['timeperiod']['timeperiod_timeranges'] as $timeRangeData) {
                $timeRangeArr[] = mb_substr($timeRangeData['weekday'], 0, 2) . ' ' .
                    $timeRangeData['start'] . ' - ' . $timeRangeData['end'];
            }
            $timeRangeText = implode(' ', $timeRangeArr);
        }
        if ($this->report['autoreport']['min_availability_percent']) {
            $minAvailabilityInPercent = floatval($this->report['autoreport']['min_availability']);
            $minAvailabilityInMin = round($this->report['autoreport_data']['total_time'] * $minAvailabilityInPercent / 6000);
        } else {
            $minAvailabilityInMin = $this->report['autoreport']['min_availability'];
            $minAvailabilityInPercent = round($minAvailabilityInMin * 6000 / $this->report['autoreport_data']['total_time'], 3);
        }

        $availabilityInPercent = strval(
            round(
                100 * ($this->report['autoreport_data']['total_time'] - $this->report['autoreport_data']['total_outage_time']) / $this->report['autoreport_data']['total_time'],
                3)
        );

        $Csv = new CsvCreator();
        $Csv->setPath($ParameterCsv);
        $Csv->addRow([
            __('Report id'),
            __('Report name'),
            __('Time periods'),
            __('Reflection state'),
            __('Downtimes'),
            __('Minimal allowed availability (min)'),
            __('From'),
            __('To'),
            __('Minimal allowed availability (%)'),
            __('Current availability'),
        ]);
        $Csv->addRow([
            $this->report['autoreport']['id'],
            $this->report['autoreport']['name'],
            $timeRangeText,
            intval($this->report['autoreport']['check_hard_state']),
            intval($this->report['autoreport']['consider_downtimes']),
            $minAvailabilityInMin,
            date('d.m.Y', $this->report['autoreport_data']['evaluation_time']['start']),
            date('d.m.Y', $this->report['autoreport_data']['evaluation_time']['end']),
            sprintf("%01.3f", $minAvailabilityInPercent),
            sprintf("%01.3f", $availabilityInPercent)
        ]);
        $Csv->closeFile();

        $Csv->setPath($DetailsCSV);
        $Csv->addRow([
            __('Report id'),
            __('Report name'),
            __('Host name'),
            __('Service name'),
            __('Outage date'),
            __('Start time'),
            __('End time'),
            __('Duration (Minutes)'),
            __('Planned downtime'),
        ]);
        foreach ($this->report['autoreport_data']['hosts'] as $hostData) {
            if (!empty($hostData['Host']['outages'])) { // host outages
                foreach ($hostData['Host']['outages'] as $outageData) {
                    $Csv->addRow([
                        $this->report['autoreport']['id'],
                        $this->report['autoreport']['name'],
                        $hostData['Host']['name'],
                        '',
                        date('d.m.Y', $outageData['start']),
                        date('H:i', $outageData['start']),
                        date('H:i', $outageData['end']),
                        round(($outageData['end'] - $outageData['start']) / 60),
                        0,
                    ]);
                }
            }
            if ($this->report['autoreport']['consider_downtimes'] && $showOutagesInDowntimes &&
                !empty($hostData['Host']['outages_summarized_in_downtime'])) { // host downtimes
                foreach ($hostData['Host']['outages_summarized_in_downtime'] as $downtimeData) {
                    $Csv->addRow([
                        $this->report['autoreport']['id'],
                        $this->report['autoreport']['name'],
                        $hostData['Host']['name'],
                        '',
                        date('d.m.Y', $downtimeData['start']),
                        date('H:i', $downtimeData['start']),
                        date('H:i', $downtimeData['end']),
                        round(($downtimeData['end'] - $downtimeData['start']) / 60),
                        1,
                    ]);
                }
            }
            if (!empty($hostData['Services'])) { // services
                foreach ($hostData['Services'] as $serviceData) {
                    if (!empty($serviceData['Service']['outages'])) { // service outages
                        foreach ($serviceData['Service']['outages'] as $outageData) {
                            $Csv->addRow([
                                $this->report['autoreport']['id'],
                                $this->report['autoreport']['name'],
                                $hostData['Host']['name'],
                                $serviceData['Service']['name'],
                                date('d.m.Y', $outageData['start']),
                                date('H:i', $outageData['start']),
                                date('H:i', $outageData['end']),
                                round(($outageData['end'] - $outageData['start']) / 60),
                                0,
                            ]);
                        }
                    }
                    if ($this->report['autoreport']['consider_downtimes'] && $showOutagesInDowntimes &&
                        !empty($serviceData['Service']['outages_summarized_in_downtime'])) { // service downtimes
                        foreach ($serviceData['Service']['outages_summarized_in_downtime'] as $downtimeData) {
                            $Csv->addRow([
                                $this->report['autoreport']['id'],
                                $this->report['autoreport']['name'],
                                $hostData['Host']['name'],
                                $serviceData['Service']['name'],
                                date('d.m.Y', $downtimeData['start']),
                                date('H:i', $downtimeData['start']),
                                date('H:i', $downtimeData['end']),
                                round(($downtimeData['end'] - $downtimeData['start']) / 60),
                                1,
                            ]);
                        }
                    }
                }
            }
        }
        $Csv->closeFile();
        $Csv->createZip($zipName, [$ParameterCsv, $DetailsCSV]);
        return $zipName;
    }
}