<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


namespace AutoreportModule\Lib;


use Cake\Filesystem\Folder;
use Cake\Utility\Hash;
use itnovum\openITCOCKPIT\Core\UUID;
use itnovum\openITCOCKPIT\Core\Views\UserTime;

class SlaChart {

    public $start_x = 90;
    public $end_x = 840;
    public $start_y = 260;
    public $end_y = 20;
    public $arrow_offset = 2;
    public $font_size = 5.8;

    /**
     * @var UserTime
     */
    private $UserTime;

    /**
     * @var int
     */
    private $start;

    /**
     * @var int
     */
    private $end;

    /**
     * @var resource
     */
    private $img;

    private $color_axis;
    private $color_text;
    private $color_vertical_grid_gray;
    private $color_vertical_red;
    private $color_horizontal_grid;
    private $color_benchmark;
    private $color_benchmark_border;
    private $color_report_data_area;
    private $color_report_data_area_border;
    private $font_file = 'DejaVuSans';
    private $max_total_time_in_seconds;
    private $axis_y_availability_in_seconds;
    private $report_interval;
    private $min_availability_in_percent;

    public function __construct(UserTime $UserTime) {
        $this->UserTime = $UserTime;
    }

    /**
     * @param $start
     * @param $end
     * @param $report_interval
     * @param $time_slices
     * @param $time_ranges
     * @param $outages
     * @param int $min_availability_in_percent
     * @return string
     * @throws \Exception
     */
    public function getReportDataAsPng($start, $end, $report_interval, $time_slices, $time_ranges, $outages, $min_availability_in_percent = 5) {
        $this->start = $start;
        $this->end = $end;
        $this->report_interval = $report_interval;

        $file_path = WWW_ROOT . 'img' . DS . 'graphs' . DS;

        $filename = UUID::v4() . '.png';
        if (!is_dir($file_path)) {
            mkdir($file_path);
        }

        $this->img = imagecreatetruecolor(870, 320);
        $this->setImageLayout(); //set transparence, colors, fonts, ...

        $interval_timestamps = $this->groupTimeSlicesByReportInteval($time_slices, $time_ranges);
        $this->min_availability_in_percent = $min_availability_in_percent;
        $this->max_total_time_in_seconds = max(Hash::map($interval_timestamps, '{n}', [$this, 'calculateTotalTime']));
        $this->axis_y_availability_in_seconds = $this->max_total_time_in_seconds * $min_availability_in_percent / 100;

        $this->createVerticalGridLines();

        $this->createHorizontalGridLines();

        $this->createBenchmarkArea($interval_timestamps, $end);
        $this->createCurrentReportDataArea($interval_timestamps, $outages);
        $this->createYAxis();
        $this->createXAxis();
        imagepng($this->img, $file_path . $filename, 0);
        imagedestroy($this->img);

        return DS . 'img' . DS . 'graphs' . DS . $filename;
    }

    public function calculateTotalTime($data) {
        return array_sum([$data['total_time'], $data['time_offset_start'], $data['time_offset_end']]);
    }

    public function createYAxis() {
        imageline($this->img, $this->start_x, $this->start_y + 4, $this->start_x, $this->end_y, $this->color_axis);
        //draw triangel (arrow) for y axis
        imagefilledpolygon($this->img, [
            $this->start_x - $this->arrow_offset, $this->end_y,
            $this->start_x + $this->arrow_offset, $this->end_y,
            $this->start_x, $this->end_y - $this->arrow_offset
        ],
            3, $this->color_axis);
    }

    public function createXAxis() {
        imageline($this->img, $this->start_x - 4, $this->start_y, $this->end_x, $this->start_y, $this->color_axis);
        //draw triangel (arrow) for x axis
        imagefilledpolygon($this->img, [
            $this->end_x, $this->start_y - $this->arrow_offset,
            $this->end_x, $this->start_y + $this->arrow_offset,
            $this->end_x + $this->arrow_offset, $this->start_y,
        ],
            3, $this->color_axis
        );
    }

    public function createVerticalGridLines() {
        $style = Hash::merge(
            array_fill(0, 6, $this->color_vertical_grid_gray),
            $this->color_vertical_red
        ); //dashed style for grig lines
        imagesetstyle($this->img, $style);

        $startDateText = $this->start_x - 18;
        $startTimeText = $this->start_x - 14;
        $grid_interval = 15;
        $interval = round(($this->end_x - $this->start_x) / $grid_interval);
        $seconds_per_interval = ($this->end - $this->start) / $grid_interval;
        for ($i = 0; $i <= $grid_interval; $i++) {
            $new_timestamp = $this->start + ($i * $seconds_per_interval);
            imageline($this->img, $this->start_x + $interval * $i, $this->end_y, $this->start_x + $interval * $i, $this->start_y, IMG_COLOR_STYLED);
            imagettftext($this->img, $this->font_size, 0, $startDateText + $interval * $i, 274, $this->color_text, $this->font_file, date('d.m.Y', $new_timestamp));
            imagettftext($this->img, $this->font_size, 0, $startTimeText + $interval * $i, 284, $this->color_text, $this->font_file, date('H:i:s', $new_timestamp));
        }
    }

    /**
     * @throws \Exception
     */
    public function createHorizontalGridLines() {
        $startYText = $this->end_y + $this->arrow_offset;
        $grid_interval = 10;
        $interval = round(($this->start_y - $this->end_y) / $grid_interval);

        $axis_y_availability_in_seconds_per_grid = $this->axis_y_availability_in_seconds / $grid_interval;
        for ($i = 0; $i <= $grid_interval; $i++) {
            $axis_y_grid_label = round($this->axis_y_availability_in_seconds - $axis_y_availability_in_seconds_per_grid * $i);
            imagedashedline($this->img, $this->start_x, $this->end_y + $interval * $i, $this->end_x, $this->end_y + $interval * $i, $this->color_horizontal_grid);
            imagettftext($this->img, $this->font_size, 0, 10, $startYText + $interval * $i, $this->color_text, $this->font_file, $this->UserTime->secondsInHumanShort($axis_y_grid_label));
        }
    }

    public function createBenchmarkArea($timeslices_per_report_interval, $end) {
        $points = [];
        $sizeofTimeSlides = sizeof($timeslices_per_report_interval);
        if (sizeof($timeslices_per_report_interval) === 1) {
            $timeslices_per_report_interval[0]['end'] = $end;
        }

        array_push($points, $this->start_x, $this->start_y); // Start Point
        foreach ($timeslices_per_report_interval as $key => $timeslice_per_report_interval) {
            $start = $timeslice_per_report_interval['start'];
            if ($key > 0) {
                array_push($points, $this->getTsX($timeslice_per_report_interval['start']), $this->start_y);
            }
            $end = $timeslice_per_report_interval['end'];
            if (($timeslice_per_report_interval['end'] > $this->end) || (
                    $timeslice_per_report_interval['end'] < $this->end && $key === ($sizeofTimeSlides) - 1)) {
                $end = $this->end;
            }
            $min_avalability_in_seconds = $this->getMinAvailabilityInSeconds($timeslice_per_report_interval['total_time']);
            $end_offset_in_seconds = $this->getMinAvailabilityInSeconds($timeslice_per_report_interval['time_offset_end']);

            array_push($points,
                $this->getTsX($start),
                $this->getTsY($min_avalability_in_seconds + $end_offset_in_seconds),
                $this->getTsX($end),
                $this->getTsY($end_offset_in_seconds)
            );
        }
        array_push($points, $this->end_x, $this->start_y);
        imagefilledpolygon($this->img, $points, sizeof($points) / 2, $this->color_benchmark);
        imagepolygon($this->img, $points, sizeof($points) / 2, $this->color_benchmark_border);
    }

    public function createCurrentReportDataArea($timeslices_per_report_interval, $outages) {
        $points = [];

        $sizeofTimeSlides = sizeof($timeslices_per_report_interval);
        array_push($points, $this->start_x, $this->start_y); // Start Point

        foreach ($timeslices_per_report_interval as $key => $timeslice_per_report_interval) {
            $start = $timeslice_per_report_interval['start'];
            $end = $timeslice_per_report_interval['end'];
            if (($timeslice_per_report_interval['end'] > $this->end) || (
                    $timeslice_per_report_interval['end'] < $this->end && $key === ($sizeofTimeSlides) - 1)) {
                $end = $this->end;
            }
            $total_time = $this->max_total_time_in_seconds;
            if ($key > 0) {
                array_push($points, $this->getTsX($timeslice_per_report_interval['start']), $this->start_y);
            }
            array_push($points,
                $this->getTsX($start),
                $this->getTsY($this->getMinAvailabilityInSeconds($total_time))
            );
            $min_avalability_in_seconds = $this->getMinAvailabilityInSeconds($total_time);
            if (!empty($outages)) {
                foreach ($outages as $outage) {
                    $outage['end'] = ($outage['end'] > $this->end) ? $this->end : $outage['end'];
                    if ($outage['start'] >= $start && $outage['end'] <= $end) {
                        $outage_duration = $outage['end'] - $outage['start'];
                        array_push($points,
                            $this->getTsX($outage['start']),
                            $this->getTsY($min_avalability_in_seconds),
                            $this->getTsX($outage['end']),
                            $this->getTsY($min_avalability_in_seconds - $outage_duration)
                        );
                        $min_avalability_in_seconds = $min_avalability_in_seconds - $outage_duration;
                    }
                }
            }
            $tmp_Y = $this->getTsY($min_avalability_in_seconds);
            array_push($points,
                $this->getTsX($end),
                $this->getTsY($min_avalability_in_seconds)
            );
        }

        array_push($points, $this->end_x, $tmp_Y);
        array_push($points, $this->end_x, $this->start_y);
        imagefilledpolygon($this->img, $points, sizeof($points) / 2, $this->color_report_data_area);
    }

    public function groupTimeSlicesByReportInteval($time_slices, $time_ranges) {
        $interval_timestamps = [];
        $check_unit = [
            'DAY'   => 'N',    //numeric representation of the day of the week; 1 (for Monday) through 7 (for Sunday)
            'WEEK'  => 'W',    //week number of year, weeks starting on Monday
            'MONTH' => 'm',    //numeric representation of a month, with leading zeros
            'YEAR'  => 'Y'    //a full numeric representation of a year, 4 digits
        ];

        $time_ranges_group_by_weekday = Hash::combine($time_ranges, '{n}.id', '{n}', '{n}.day');
        if ($this->report_interval != 'DAY' && $this->report_interval != 'QUARTER') {
            $tmp_week_day = null;
            $intern_counter = 0;
            $new_slice_created = false;
            //Create global time slices for benchmark chart area (orange area)
            foreach ($time_slices as $key => $time_slice) {
                if ($tmp_week_day != date($check_unit[$this->report_interval], $time_slice['start'])) {
                    $tmp_week_day = date($check_unit[$this->report_interval], $time_slice['start']);
                    if (date($check_unit[$this->report_interval], $time_slice['start']) == date($check_unit[$this->report_interval], $this->start) && !$new_slice_created) {
                        //check if week|month|year == (week|month|year) $this->start
                        $interval_timestamps[$intern_counter]['start'] = $this->start;

                    } else {
                        if ($new_slice_created) {
                            switch ($this->report_interval) {
                                case 'WEEK':
                                    $start = strtotime('this monday' . date('d.m.Y', $time_slice['start']) . ' 00:00:00');
                                    break;
                                case 'MONTH':
                                    $start = strtotime('01.' . date('m.Y', $time_slice['start']) . ' 00:00:00');
                                    break;
                                case 'YEAR':
                                    $start = strtotime('01.01.' . date('Y', $time_slice['start']) . ' 00:00:00');
                                    break;
                            }
                            $interval_timestamps[$intern_counter]['start'] = $start;
                            $new_slice_created = false;
                        }
                    }
                    $interval_timestamps[$intern_counter]['total_time'] = 0;
                }
                if (isset($time_slices[$key + 1])) {
                    if ($tmp_week_day != date($check_unit[$this->report_interval], $time_slices[$key + 1]['start'])) {
                        $interval_timestamps[$intern_counter]['end'] = $time_slice['end'];
                        $interval_timestamps[$intern_counter]['time_offset_start'] = 0;
                        $interval_timestamps[$intern_counter]['time_offset_end'] = 0;
                        $interval_timestamps[$intern_counter]['total_time'] += $time_slice['end'] - $time_slice['start'];
                        $intern_counter++;
                        $new_slice_created = true;
                    } else {
                        $interval_timestamps[$intern_counter]['total_time'] += $time_slice['end'] - $time_slice['start'];
                    }
                } else {
                    $end = ($time_slice['end'] > $this->end) ? $this->end : $time_slice['end'];
                    $interval_timestamps[$intern_counter]['end'] = $end;
                    $interval_timestamps[$intern_counter]['time_offset_start'] = 0;
                    $interval_timestamps[$intern_counter]['time_offset_end'] = 0;
                    $interval_timestamps[$intern_counter]['total_time'] += $end - $time_slice['start'];
                }
            }
            //time offset if first time slice not completely
            $first_key_in_time_slices = 0;
            switch ($this->report_interval) {
                case 'WEEK':
                    $first_day_in_week = 1; //7 (for Monday)
                    $current_week_day = date('N', $interval_timestamps[$first_key_in_time_slices]['start']);
                    if ($current_week_day !== $first_day_in_week) {
                        $offset_days = Hash::extract($time_ranges, '{n}[day<' . $current_week_day . ']');
                        foreach ($offset_days as $offset_day) {
                            $interval_timestamps[$first_key_in_time_slices]['time_offset_start'] += strtotime($offset_day['end']) - strtotime($offset_day['start']);
                        }
                    }
                    break;
                case 'MONTH':
                    $first_day_in_month = 1;
                    if ($first_day_in_month != date('j', $interval_timestamps[$first_key_in_time_slices]['start'])) {
                        $offset_days_count = date('j', $interval_timestamps[$first_key_in_time_slices]['end'] - 1);
                        for ($i = 1; $i <= $offset_days_count; $i++) {
                            $current_weekday = date('N', strtotime(date('d.m.Y H:i:s', $interval_timestamps[$first_key_in_time_slices]['start']) . ' +' . $i . ' days'));
                            if (array_key_exists($current_weekday, $time_ranges_group_by_weekday)) {
                                foreach ($time_ranges_group_by_weekday[$current_weekday] as $week_day => $offset_day) {
                                    $interval_timestamps[$first_key_in_time_slices]['time_offset_start'] += strtotime($offset_day['end']) - strtotime($offset_day['start']);
                                }
                            }
                        }
                    }
                    break;
                case 'YEAR':
                    $first_day_in_year = 0;
                    $time_ranges_group_by_weekday = Hash::combine($time_ranges, '{n}.id', '{n}', '{n}.day');
                    if ($first_day_in_year != date('z', $interval_timestamps[$first_key_in_time_slices]['start'])) {
                        $offset_days_count = date('z', $interval_timestamps[$first_key_in_time_slices]['start']);

                        for ($i = 1; $i <= $offset_days_count; $i++) {
                            $current_weekday = date('N', strtotime(date('d.m.Y H:i:s', $interval_timestamps[$first_key_in_time_slices]['start']) . ' +' . $i . ' days'));
                            if (array_key_exists($current_weekday, $time_ranges_group_by_weekday)) {
                                foreach ($time_ranges_group_by_weekday[$current_weekday] as $week_day => $offset_day) {
                                    $interval_timestamps[$first_key_in_time_slices]['time_offset_start'] += strtotime($offset_day['end']) - strtotime($offset_day['start']);
                                }
                            }
                        }
                    }
                    break;
            }
            //time offset if last time slice not completely
            $last_key_in_time_slices = sizeof($interval_timestamps) - 1;
            switch ($this->report_interval) {
                case 'WEEK':
                    $last_day_in_week = 7; //7 (for Sunday)
                    $current_week_day = date('N', $interval_timestamps[$last_key_in_time_slices]['end']);
                    if ($current_week_day !== $last_day_in_week) {
                        $offset_days = Hash::extract($time_ranges, '{n}[day>' . $current_week_day . ']');
                        foreach ($offset_days as $offset_day) {
                            $interval_timestamps[$last_key_in_time_slices]['time_offset_end'] += strtotime($offset_day['end']) - strtotime($offset_day['start']);
                        }
                    }
                    $interval_timestamps[$last_key_in_time_slices]['end'] = strtotime('this sunday' . date('d.m.o', $time_slice['end']) . ' 23:59:59');
                    break;
                case 'MONTH':
                    $last_day_in_month = date('t', $interval_timestamps[$last_key_in_time_slices]['end']);
                    $time_ranges_group_by_weekday = Hash::combine($time_ranges, '{n}.id', '{n}', '{n}.day');
                    if ($last_day_in_month != date('j', $interval_timestamps[$last_key_in_time_slices]['end'])) {
                        $offset_days_count = $last_day_in_month - date('j', $interval_timestamps[$last_key_in_time_slices]['end']);
                        for ($i = 1; $i <= $offset_days_count; $i++) {
                            $current_weekday = date('N', strtotime(date('d.m.o H:i:s', $interval_timestamps[$last_key_in_time_slices]['end']) . ' +' . $i . ' days'));
                            if (array_key_exists($current_weekday, $time_ranges_group_by_weekday)) {
                                foreach ($time_ranges_group_by_weekday[$current_weekday] as $week_day => $offset_day) {
                                    $interval_timestamps[$last_key_in_time_slices]['time_offset_end'] += strtotime($offset_day['end']) - strtotime($offset_day['start']);
                                }
                            }
                        }
                    }
                    break;
                case 'YEAR':
                    $last_day_in_year = (date('L', $interval_timestamps[$last_key_in_time_slices]['end'])) ? 366 : 365;
                    if ($last_day_in_year != date('z', $interval_timestamps[$last_key_in_time_slices]['end'])) {
                        $offset_days_count = $last_day_in_year - date('z', $interval_timestamps[$last_key_in_time_slices]['end']);
                        for ($i = 1; $i <= $offset_days_count; $i++) {
                            $current_weekday = date('N', strtotime(date('d.m.o H:i:s', $interval_timestamps[$last_key_in_time_slices]['end']) . ' +' . $i . ' days'));
                            if (array_key_exists($current_weekday, $time_ranges_group_by_weekday)) {
                                foreach ($time_ranges_group_by_weekday[$current_weekday] as $week_day => $offset_day) {
                                    $interval_timestamps[$last_key_in_time_slices]['time_offset_end'] += strtotime($offset_day['end']) - strtotime($offset_day['start']);
                                }
                            }
                        }
                    }
                    break;
            }
        } else if ($this->report_interval === 'QUARTER') {
            $tmp_current_quarter = null;
            $intern_counter = 0;
            $new_slice_created = false;
            //Create global time slices for benchmark chart area (orange area)
            foreach ($time_slices as $key => $time_slice) {
                $currentQuarter = ceil(date('n', $time_slice['start']) / 3);
                if ($tmp_current_quarter != $currentQuarter) {
                    $tmp_current_quarter = $currentQuarter;
                    $currentQuarterThisStart = ceil(date('n', $this->start) / 3);
                    if ($currentQuarter == $currentQuarterThisStart && !$new_slice_created) {
                        //check if week|month|year == (week|month|year) $this->start
                        $interval_timestamps[$intern_counter]['start'] = $this->start;
                    } else {
                        if ($new_slice_created) {
                            $month = date('n', $time_slice['start']);
                            if ($month <= 3) {
                                $start = strtotime('01.01.' . date('Y', $time_slice['start']) . ' 00:00:00');
                            } else if ($month <= 6) {
                                $start = strtotime('01.04.' . date('Y', $time_slice['start']) . ' 00:00:00');
                            } else if ($month <= 9) {
                                $start = strtotime('01.07.' . date('Y', $time_slice['start']) . ' 00:00:00');
                            } else {
                                $start = strtotime('01.10.' . date('Y', $time_slice['start']) . ' 00:00:00');
                            }
                            $interval_timestamps[$intern_counter]['start'] = $start;
                            $new_slice_created = false;
                        }
                    }
                    $interval_timestamps[$intern_counter]['total_time'] = 0;
                }
                if (isset($time_slices[$key + 1])) {
                    $currentQuarterNext = ceil(date('n', $time_slices[$key + 1]['start']) / 3);
                    if ($tmp_current_quarter != $currentQuarterNext) {
                        $interval_timestamps[$intern_counter]['end'] = $time_slice['end'];
                        $interval_timestamps[$intern_counter]['time_offset_start'] = 0;
                        $interval_timestamps[$intern_counter]['time_offset_end'] = 0;
                        $interval_timestamps[$intern_counter]['total_time'] += $time_slice['end'] - $time_slice['start'];
                        $intern_counter++;
                        $new_slice_created = true;
                    } else {
                        $interval_timestamps[$intern_counter]['total_time'] += $time_slice['end'] - $time_slice['start'];
                    }
                } else {
                    $end = ($time_slice['end'] > $this->end) ? $this->end : $time_slice['end'];
                    $interval_timestamps[$intern_counter]['end'] = $end;
                    $interval_timestamps[$intern_counter]['time_offset_start'] = 0;
                    $interval_timestamps[$intern_counter]['time_offset_end'] = 0;
                    $interval_timestamps[$intern_counter]['total_time'] += $end - $time_slice['start'];
                }
            }
            //time offset if first time slice not completely
            $first_key_in_time_slices = 0;
            $month = date('n', $interval_timestamps[$first_key_in_time_slices]['start']);
            if ($month <= 3) {
                $first_day_in_quarter = strtotime('01.01.' . date('o', $time_slice['start']) . ' 00:00:00');
            } else if ($month <= 6) {
                $first_day_in_quarter = strtotime('01.04.' . date('Y', $time_slice['start']) . ' 00:00:00');
            } else if ($month <= 9) {
                $first_day_in_quarter = strtotime('01.07.' . date('Y', $time_slice['start']) . ' 00:00:00');
            } else {
                $first_day_in_quarter = strtotime('01.10.' . date('Y', $time_slice['start']) . ' 00:00:00');
            }
            $first_day_in_quarter = date('z', $first_day_in_quarter);
            $time_ranges_group_by_weekday = Hash::combine($time_ranges, '{n}.id', '{n}', '{n}.day');
            if ($first_day_in_quarter != date('z', $interval_timestamps[$first_key_in_time_slices]['start'])) {
                $offset_days_count = date('z', $interval_timestamps[$first_key_in_time_slices]['start']) - $first_day_in_quarter;
                for ($i = 1; $i <= $offset_days_count; $i++) {
                    $current_weekday = date('N', strtotime(date('d.m.o H:i:s', $interval_timestamps[$first_key_in_time_slices]['start']) . ' +' . $i . ' days'));
                    if (array_key_exists($current_weekday, $time_ranges_group_by_weekday)) {
                        foreach ($time_ranges_group_by_weekday[$current_weekday] as $week_day => $offset_day) {
                            $interval_timestamps[$first_key_in_time_slices]['time_offset_start'] += strtotime($offset_day['end']) - strtotime($offset_day['start']);
                        }
                    }
                }
            }
            //time offset if last time slice not completely
            $last_key_in_time_slices = sizeof($interval_timestamps) - 1;
            $month = date('n', $interval_timestamps[$last_key_in_time_slices]['end']);
            if ($month <= 3) {
                $last_day_in_quarter = strtotime('31.03.' . date('Y', $time_slice['end']) . ' 23:59:59');
            } else if ($month <= 6) {
                $last_day_in_quarter = strtotime('30.06.' . date('Y', $time_slice['end']) . ' 23:59:59');
            } else if ($month <= 9) {
                $last_day_in_quarter = strtotime('30.09.' . date('Y', $time_slice['end']) . ' 23:59:59');
            } else {
                $last_day_in_quarter = strtotime('31.12.' . date('o', $time_slice['end']) . ' 23:59:59');
            }
            $last_day_in_quarter = date('z', $last_day_in_quarter);
            if (date('z', $last_day_in_quarter) != date('z', $interval_timestamps[$last_key_in_time_slices]['end'])) {
                $offset_days_count = $last_day_in_quarter - date('z', $interval_timestamps[$last_key_in_time_slices]['end']);
                for ($i = 1; $i <= $offset_days_count; $i++) {
                    $current_weekday = date('N', strtotime(date('d.m.o H:i:s', $interval_timestamps[$last_key_in_time_slices]['end']) . ' +' . $i . ' days'));
                    if (array_key_exists($current_weekday, $time_ranges_group_by_weekday)) {
                        foreach ($time_ranges_group_by_weekday[$current_weekday] as $week_day => $offset_day) {
                            $interval_timestamps[$last_key_in_time_slices]['time_offset_end'] += strtotime($offset_day['end']) - strtotime($offset_day['start']);
                        }
                    }
                }
            }
        } else {
            $tmp_end_date = $this->start;
            $intern_counter = 0;
            while ($tmp_end_date < $this->end) {
                $tmp_start_date = strtotime(date('d.m.o', $this->start) . ' 00:00:00 + ' . $intern_counter . ' day ');
                $tmp_end_date = strtotime(date('d.m.o', $this->start) . ' 23:59:59 + ' . $intern_counter . ' day ');
                $current_weekday = date('N', $tmp_start_date);
                $interval_timestamps[$intern_counter]['start'] = $tmp_start_date;
                $interval_timestamps[$intern_counter]['end'] = $tmp_end_date;
                $interval_timestamps[$intern_counter]['time_offset_start'] = 0;
                $interval_timestamps[$intern_counter]['time_offset_end'] = 0;
                $interval_timestamps[$intern_counter]['total_time'] = 0;
                if (array_key_exists($current_weekday, $time_ranges_group_by_weekday)) {
                    foreach ($time_ranges_group_by_weekday[$current_weekday] as $week_day => $offset_day) {
                        $interval_timestamps[$intern_counter]['total_time'] += strtotime($offset_day['end']) - strtotime($offset_day['start']);
                    }
                }
                $intern_counter++;
            }

        }
        return $interval_timestamps;
    }

    public function getTsX($timestamp) {
        $graph_width = $this->end_x - $this->start_x;
        $total_graph_time = $this->end - $this->start;
        $timestamp_diff_percent = ($timestamp - $this->start) / $total_graph_time * 100;
        $x_position = round($this->start_x + $graph_width * $timestamp_diff_percent / 100);

        return $x_position;
    }

    public function getTsY($seconds) {
        $graph_height = $this->start_y - $this->end_y;
        $seconds_diff_in_percent = $seconds / $this->axis_y_availability_in_seconds * 100;
        $y_position = round($this->start_y - $seconds_diff_in_percent * $graph_height / 100);

        return ($y_position > $this->start_y) ? $this->start_y : $y_position;
    }

    public function getMinAvailabilityInSeconds($total_time) {
        return (int)$total_time * $this->min_availability_in_percent / 100;
    }

    public function setImageLayout() {
        imagesavealpha($this->img, true);
        $transparent = imagecolorallocatealpha($this->img, 0, 0, 0, 127);
        imagefill($this->img, 0, 0, $transparent);

        /*** colors definition ***/
        /*** colors for x ans y axis ***/
        $this->color_axis = imagecolorallocatealpha($this->img, 76, 79, 83, 40);
        /*** colors for text output ***/
        $this->color_text = imagecolorallocate($this->img, 76, 79, 83);
        /*** colors for vertical grid lines ***/
        $this->color_vertical_grid_gray = imagecolorallocatealpha($this->img, 144, 149, 155, 80);
        $this->color_vertical_red = imagecolorallocate($this->img, 255, 0, 0);
        /*** colors for horizontal grid lines ***/
        $this->color_horizontal_grid = imagecolorallocatealpha($this->img, 144, 149, 155, 80);
        /*** colors for benchmark area (orange) ***/
        $this->color_benchmark = imagecolorallocatealpha($this->img, 204, 0, 0, 40);
        $this->color_benchmark_border = imagecolorallocatealpha($this->img, 204, 0, 0, 30);
        /*** colors for report data area (green) ***/
        $this->color_report_data_area = imagecolorallocatealpha($this->img, 0, 200, 81, 55);
        $this->color_report_data_area_border = imagecolorallocatealpha($this->img, 0, 200, 81, 20);

        putenv('GDFONTPATH=/usr/share/fonts/truetype/dejavu/');
        $this->font_file = 'DejaVuSans';
    }
}