<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace AutoreportModule\Model\Table;

use Cake\Datasource\RepositoryInterface;
use Cake\ORM\Table;
use Cake\Utility\Hash;

class AutoreportsHostsMembershipsTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setEntityClass('AutoreportModule.AutoreportHostMembership');
        $this->setTable('hosts_to_autoreports');

        $this->belongsTo('AutoreportModule.Autoreports');
        $this->belongsTo('Hosts');
    }

    public function bindCoreAssociations(RepositoryInterface $coreTable) {
        switch ($coreTable->getAlias()) {
            case 'Hosts':
                $coreTable->hasMany('AutoreportModule.AutoreportsHostsMemberships', [
                    'dependent'  => true,
                    'foreignKey' => 'host_id'
                ]);
                break;
        }
    }

    /**
     * @param int $excludedReportId
     * @param int $hostId
     * @return int
     */
    public function isHostUsedByReport($excludedReportId, $hostId) {
        $count = $this->find()
            ->where([
                'autoreport_id !=' => $excludedReportId,
                'host_id'          => $hostId
            ])
            ->count();

        return $count;
    }

    /**
     * @param int $hostId
     * @return array
     */
    public function getAllAutoreportIdsByHostId($hostId) {
        $result = $this->find()
            ->select([
                'autoreport_id'
            ])
            ->where([
                'host_id' => $hostId
            ])
            ->disableHydration()
            ->all();

        if ($result === null) {
            return [];
        }

        return Hash::extract($result->toArray(), '{n}.autoreport_id');
    }
}