<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement. 
// The license agreement and license key were sent with the order confirmation.

/**
 * @var \App\View\AppView $this
 * @var bool $success
 * @var array $report
 * @var \itnovum\openITCOCKPIT\Core\Views\UserTime $UserTime
 *
 */

use Cake\Utility\Hash;
use itnovum\openITCOCKPIT\Core\Views\Logo;

$Logo = new Logo();
$css = \App\itnovum\openITCOCKPIT\Core\AngularJS\PdfAssets::getCssFiles();
?>
<head>
    <?php
    foreach ($css as $cssFile): ?>
        <link rel="stylesheet" type="text/css" href="<?= WWW_ROOT . $cssFile; ?>"/>
    <?php
    endforeach; ?>

    <style>
        <?= file_get_contents(PLUGIN . 'AutoreportModule' . DS . 'webroot' . DS . 'css/autoreports_pdf.css'); ?>
    </style>


</head>
<body class="">

<div class="row">
    <div class="col-6 padding-left-15 font-lg">
        <i class="fa fa-file-invoice" style="font-size: 20px!important;"></i>
        <?php echo h($report['autoreport']['name']); ?>
        <?php if (!empty($autoreport['autoreport']['description'])): ?>
            <em><?= h($autoreport['autoreport']['description']) ?></em>
        <?php endif; ?>
    </div>
    <div class="col-6">
        <img class="float-right" src="<?php echo $Logo->getLogoPdfPath(); ?>" width="200"/>
    </div>
</div>
<div class="row">
    <div class="col-4">
        <h4>
            <?= __('Analysis period'); ?>
        </h4>
    </div>
    <div class="col-8">
        <?php
        echo h($reportStart); ?>
        <i class="fas fa-long-arrow-alt-right"></i>
        <?php
        echo h($reportEnd);
        ?>

    </div>
</div>

<div class="row padding-top-10">
    <div class="col-4">
        <h4>
            <?= __('Evaluated time period'); ?>
        </h4>
    </div>

    <div class="col-8">
        <div class="row">
            <div class="col-12">
                <?= h($report['autoreport']['timeperiod']['name']) ?>
            </div>
        </div>
        <?php foreach ($report['autoreport']['timeperiod']['timeperiod_timeranges'] as $timerange): ?>
            <div class="row">
                <div class="col-2">
                    <?= h($timerange['weekday']); ?>
                </div>
                <div class="col-10">
                    <?= h($timerange['start']); ?>
                    -
                    <?= h($timerange['end']); ?>
                </div>
            </div>
        <?php endforeach; ?>
    </div>
</div>

<?php if (($report['autoreport_data']['settings']['configuration_option'] & $report['autoreport_data']['defaultSettings']['AUTOREPORTS_SHOW_SUMMARY_STATISTICS']['value']) && $report['autoreport']['min_availability']): ?>
    <div class="row padding-top-10">
        <div class="col-12">
            <h4>
                <?= __('Availability overview') ?>
            </h4>
        </div>
    </div>
    <div class="row">
        <div class="col-12">
            <table class="table ">
                <thead>
                <tr>
                    <th style="border-top:none;"></th>
                    <th style="border-top:none;">
                        <?= __('Maximum allowed outage duration') ?>
                    </th>
                    <th style="border-top:none;">
                        <?= __('Measured outage duration ') ?>
                    </th>
                    <th style="border-top:none; width: 100px;"></th>
                </tr>
                </thead>
                <tbody>
                <tr>
                    <td><?= __('Percentage') ?></td>
                    <td>
                        <?= round($report['autoreport_data']['max_outages_time_in_percent'], 3) ?> %
                    </td>
                    <td>
                        <?= round($report['autoreport_data']['total_outage_time_percentage'], 3) ?> %
                    </td>
                    <td>
                        <?php if ($report['autoreport_data']['total_outage_time_percentage'] <= $report['autoreport_data']['max_outages_time_in_percent']): ?>
                            <span class="badge border border-success text-success">
                                <?= __('PASSED') ?>
                            </span>
                        <?php else: ?>
                            <span class="badge border border-danger text-danger">
                                <?= __('FAILED') ?>
                            </span>
                        <?php endif; ?>
                    </td>
                </tr>
                <tr>
                    <td><?= __('Minutes') ?></td>
                    <td>
                        <?= h($report['autoreport_data']['max_outages_time_in_seconds_human']) ?>
                    </td>
                    <td>
                        <?= h($report['autoreport_data']['total_outage_time_human']) ?>
                    </td>
                    <td>
                        <?php if ($report['autoreport_data']['total_outage_time_percentage'] <= $report['autoreport_data']['max_outages_time_in_percent']): ?>
                            <span class="badge border border-success text-success">
                                <?= __('PASSED') ?>
                            </span>
                        <?php else: ?>
                            <span class="badge border border-danger text-danger">
                                <?= __('FAILED') ?>
                            </span>
                        <?php endif; ?>
                    </td>
                </tr>
                </tbody>
            </table>
        </div>
    </div>
<?php endif; ?>

<?php if (($report['autoreport_data']['settings']['configuration_option'] & $report['autoreport_data']['defaultSettings']['AUTOREPORTS_SHOW_SUMMARY_STATISTICS']['value']) && $report['autoreport']['max_number_of_outages']): ?>
    <div class="row padding-top-10">
        <div class="col-12">
            <h4>
                <?= __('Outages overview') ?>
            </h4>
        </div>
    </div>
    <div class="row">
        <div class="col-12">
            <table class="table ">
                <thead>
                <tr>
                    <th style="border-top:none;">
                        <?= __('Maximum allowed number of outages') ?>
                    </th>
                    <th style="border-top:none;">
                        <?= __('Measured number of outages') ?>
                    </th>
                    <th style="border-top:none; width: 100px;"></th>
                </tr>
                </thead>
                <tbody>
                <tr>
                    <td>
                        <?= h($report['autoreport']['max_number_of_outages']) ?>
                    </td>
                    <td>
                        <?= h($report['autoreport_data']['total_outage_count']) ?>
                    </td>
                    <td>
                        <?php if ($report['autoreport_data']['total_outage_count'] <= $report['autoreport']['max_number_of_outages']): ?>
                            <span class="badge border border-success text-success">
                                <?= __('PASSED') ?>
                            </span>
                        <?php else: ?>
                            <span class="badge border border-danger text-danger">
                                <?= __('FAILED') ?>
                            </span>
                        <?php endif; ?>
                    </td>
                </tr>
                </tbody>
            </table>
        </div>
    </div>
<?php endif; ?>

<div class="row padding-top-10">
    <div class="col-12">
        <h4>
            <?= __('Evaluated host and services') ?>
        </h4>
    </div>
</div>
<div class="row">
    <div class="col-12">
        <table class="table ">
            <thead>
            <tr>
                <th style="border-top:none;">
                    <?= __('Name') ?>
                </th>
                <th style="border-top:none;">
                    <?php if ($report['autoreport_data']['settings']['configuration_option'] & $report['autoreport_data']['defaultSettings']['AUTOREPORTS_EXTENDED_HOST_DESC']['value'] || $report['autoreport_data']['settings']['configuration_option'] & $report['autoreport_data']['defaultSettings']['AUTOREPORTS_EXTENDED_SERV_DESC']['value']): ?>
                        <?= __('Availability ') ?>
                    <?php endif; ?>
                </th>
                <th style="border-top:none; width: 100px;"></th>
            </tr>
            </thead>
            <tbody>
            <?php foreach ($report['autoreport_data']['hosts'] as $host): ?>
                <tr>
                    <td style="background: #DFDFDF!important;">
                        <i class="fas fa-desktop padding-right-5"></i>
                        <?= h($host['Host']['name']) ?>
                    </td>
                    <td style="background: #DFDFDF!important;">
                        <?php if (!empty($host['Host']['state_overview'])): ?>
                            <?php if ($report['autoreport_data']['settings']['configuration_option'] & $report['autoreport_data']['defaultSettings']['AUTOREPORTS_EXTENDED_HOST_DESC']['value']): ?>
                                <?= round($host['Host']['state_overview']['percent_availability'], 3) ?> %
                            <?php endif; ?>
                        <?php elseif (!empty(Hash::get($report, 'hosts.{n}.[id=' . $host['Host']['id'] . ']'))): ?>
                            <i class="text-primary">
                                <?= __('No historical data available '); ?>
                            </i>
                        <?php endif; ?>
                    </td>
                    <td style="background: #DFDFDF!important;">
                        <?php if (!empty($host['Host']['state_overview'])): ?>
                            <?php if ($report['autoreport_data']['settings']['configuration_option'] & $report['autoreport_data']['defaultSettings']['AUTOREPORTS_EXTENDED_HOST_DESC']['value']): ?>
                                <?php if (100 - $report['autoreport_data']['max_outages_time_in_percent'] <= $host['Host']['state_overview']['percent_availability']): ?>
                                    <span class="badge border border-success text-success">
                                        <?= __('PASSED') ?>
                                    </span>
                                <?php else: ?>
                                    <span class="badge border border-danger text-danger">
                                        <?= __('FAILED') ?>
                                    </span>
                                <?php endif; ?>
                            <?php endif; ?>
                        <?php endif; ?>
                    </td>
                </tr>

                <?php foreach ($host['Services'] as $service): ?>
                    <tr>
                        <td>
                            <span class="padding-left-10">
                                <i class="fas fa-cog padding-right-5"></i>
                                <?= h($service['Service']['name']) ?>
                            </span>
                        </td>
                        <td>
                            <?php if (!empty($service['Service']['state_overview'])): ?>
                                <?php if ($report['autoreport_data']['settings']['configuration_option'] & $report['autoreport_data']['defaultSettings']['AUTOREPORTS_EXTENDED_SERV_DESC']['value']): ?>
                                    <?= round($service['Service']['state_overview']['percent_availability'], 3) ?> %
                                <?php endif; ?>
                            <?php else: ?>
                                <i class="text-primary">
                                    <?= __('No historical data available '); ?>
                                </i>
                            <?php endif; ?>
                        </td>
                        <td>
                            <?php if (!empty($service['Service']['state_overview'])): ?>
                                <?php if ($report['autoreport_data']['settings']['configuration_option'] & $report['autoreport_data']['defaultSettings']['AUTOREPORTS_EXTENDED_SERV_DESC']['value']): ?>
                                    <?php if (100 - $report['autoreport_data']['max_outages_time_in_percent'] <= $service['Service']['state_overview']['percent_availability']): ?>
                                        <span class="badge border border-success text-success">
                                            <?= __('PASSED') ?>
                                        </span>
                                    <?php else: ?>
                                        <span class="badge border border-danger text-danger">
                                            <?= __('FAILED') ?>
                                        </span>
                                    <?php endif; ?>
                                <?php endif; ?>
                            <?php endif; ?>
                        </td>
                    </tr>
                <?php endforeach; ?>
            <?php endforeach; ?>
            </tbody>
        </table>
    </div>
</div>

<?php if ($report['autoreport_data']['sla_graph_path']): ?>
    <div class="row">
        <div class="col-4">
            <h4>
                <?= __('SLA overview'); ?>
            </h4>
        </div>
    </div>
    <div class="row">
        <div class="col-12 text-center">
            <img src="<?= WWW_ROOT . $report['autoreport_data']['sla_graph_path']; ?>" class="img-fluid"
                 alt="SLA Chart">
        </div>
    </div>
<?php endif; ?>

<div class="row">
    <div class="col-4">
        <h4>
            <?= __('Host and service details'); ?>
        </h4>
    </div>
</div>
<div class="row">
    <div class="col-12">
        <?php foreach ($report['autoreport_data']['hosts'] as $host): ?>
            <div class="row html-report-host-row padding-top-10 padding-bottom-10 border-top"
                 style="background: #DFDFDF!important;">

                <div class="col-3 text-left">
                    <i class="fas fa-desktop padding-left-5 padding-right-5"></i>
                    <?= h($host['Host']['name']); ?>
                    <?php if (isset($host['Host']['configuration_option']) && ($host['Host']['configuration_option'] & 4)
                        && strlen($host['Host']['description']) > 0): ?>
                        <!-- 4 == alias -->
                        - <?= h($host['Host']['description']) ?>
                    <?php endif; ?>
                </div>

                <div class="col-2">
                    <?php if (!empty($host['Host']['state_overview'])): ?>
                        <div class="report-border-sucess padding-left-2">
                            <?php if ($host['Host']['configuration_option'] & 1): ?>
                                <div class="row">
                                    <div class="col-12">
                                        <!-- 1 == percent -->
                                        <?= round($host['Host']['state_overview']['percent_up'], 3) ?> %
                                    </div>
                                </div>
                            <?php endif; ?>
                            <?php if ($host['Host']['configuration_option'] & 2): ?>
                                <div class="row">
                                    <div class="col-12">
                                        <!-- 2 == minute -->
                                        <?= h($host['Host']['state_overview']['human_up']) ?>
                                    </div>
                                </div>
                            <?php endif; ?>
                        </div>
                    <?php endif; ?>
                </div>

                <div class="col-2">
                    <?php if (!empty($host['Host']['state_overview'])): ?>
                        <div class="report-border-danger padding-left-2">
                            <?php if ($host['Host']['configuration_option'] & 1): ?>
                                <div class="row">
                                    <div class="col-12">
                                        <!-- 1 == percent -->
                                        <?= round($host['Host']['state_overview']['percent_down'], 3) ?> %
                                    </div>
                                </div>
                            <?php endif; ?>
                            <?php if ($host['Host']['configuration_option'] & 2): ?>
                                <div class="row">
                                    <div class="col-12">
                                        <!-- 2 == minute -->
                                        <?= h($host['Host']['state_overview']['human_down']) ?>
                                    </div>
                                </div>
                            <?php endif; ?>
                        </div>
                    <?php endif; ?>
                </div>

                <div class="col-2">
                    <?php if (!empty($host['Host']['state_overview'])): ?>
                        <div class="report-border-default padding-left-2">
                            <?php if ($host['Host']['configuration_option'] & 1): ?>
                                <div class="row">
                                    <div class="col-12">
                                        <!-- 1 == percent -->
                                        <?= round($host['Host']['state_overview']['percent_unreachable'], 3) ?> %
                                    </div>
                                </div>
                            <?php endif; ?>
                            <?php if ($host['Host']['configuration_option'] & 2): ?>
                                <div class="row">
                                    <div class="col-12">
                                        <!-- 2 == minute -->
                                        <?= h($host['Host']['state_overview']['human_unreachable']) ?>
                                    </div>
                                </div>
                            <?php endif; ?>
                        </div>
                    <?php endif; ?>
                </div>

                <div class="col-3">
                    <!-- empty space for service status -->
                </div>
            </div>

            <!-- Outages -->
            <?php if (!empty($host['Host']['frontend']['outages'])): ?>
                <?php if (sizeof($host['Host']['frontend']['outages']) > 0): ?>
                    <div class="row html-report-host-row padding-top-5"
                         style="background: #fbfbfb!important;">
                        <!-- host outages -->
                        <div class="col-3"><!-- spacer --></div>
                        <div class="col-4">
                        </div>
                        <div class="col-5 report-font-weight-500">
                            <?php if ($host['Host']['outage_duration'] !== null): ?>
                                <b><?= __('Outage time exceeding the permitted limit ({0} min)', $host['Host']['outage_duration']) ?></b>
                            <?php endif; ?>
                        </div>
                    </div>

                    <?php foreach ($host['Host']['frontend']['outages'] as $outage): ?>
                        <div class="row" style="padding: 0; background: #fbfbfb!important;">
                            <?php
                            $style = '';
                            if ($outage['duration'] === $host['Host']['maximum_outage_duration']):
                                $style = 'border: 1px solid #DEE3E5!important; background: #DEE3E5!important;';
                            endif;
                            ?>
                            <div class="col-3" style="<?= $style ?>"><!-- spacer --></div>
                            <div class="col-4" style="<?= $style ?>">
                                <?= h($outage['start']) ?>
                                -
                                <?= h($outage['end']) ?>
                            </div>
                            <div class="col-5" style="<?= $style ?>">
                                <?= h($outage['duration_str']) ?>
                            </div>
                        </div>
                    <?php endforeach; ?>
                    <div class="row padding-bottom-10" style="background: #fbfbfb!important;"></div>
                <?php endif; ?>
            <?php endif; ?>
            <!-- Outages in downtimes -->
            <?php if (!empty($host['Host']['frontend']['outages_summarized_in_downtime'])): ?>
                <?php if (sizeof($host['Host']['frontend']['outages_summarized_in_downtime']) > 0): ?>
                    <div class="row html-report-host-row padding-top-5"
                         style="background: #fbfbfb!important;">
                        <!-- host outages -->
                        <div class="col-3"><!-- spacer --></div>
                        <div class="col-9">
                            <b>
                                <i class="far fa-clock"></i>
                                <?php if ($host['Host']['outage_duration']): ?>
                                    <?= __('Host outage in downtime exceeding the permitted limit ({0} min)', $host['Host']['outage_duration']) ?>
                                <?php else: ?>
                                    <?= __('Host outage in downtime') ?>
                                <?php endif; ?>
                            </b>
                        </div>
                    </div>


                    <?php foreach ($host['Host']['frontend']['outages_summarized_in_downtime'] as $outage): ?>
                        <div class="row" style="padding: 0; background: #fbfbfb!important;">
                            <div class="col-3"><!-- spacer --></div>
                            <div class="col-4">
                                <?= h($outage['start']) ?>
                                -
                                <?= h($outage['end']) ?>
                            </div>
                            <div class="col-5">
                                <?= h($outage['duration_str']) ?>
                            </div>
                        </div>
                    <?php endforeach; ?>
                    <div class="row padding-bottom-10" style="background: #fbfbfb!important;"></div>
                <?php endif; ?>
            <?php endif; ?>
            <?php if (!empty($host['Host']['frontend']['downtimes'])): ?>
                <?php if (sizeof($host['Host']['frontend']['downtimes']) > 0): ?>
                    <div class="row html-report-host-row padding-top-5"
                         style="background: #fbfbfb!important;">
                        <!-- host outages -->
                        <div class="col-3"><!-- spacer --></div>
                        <div class="col-9">
                            <b>
                                <i class="fas fa-power-off"></i>
                                <?= __('Host downtimes') ?>
                            </b>
                        </div>
                    </div>

                    <?php foreach ($host['Host']['frontend']['downtimes'] as $downtime): ?>
                        <div class="row" style="padding: 0; background: #fbfbfb!important;">
                            <div class="col-3"><!-- spacer --></div>
                            <div class="col-4">
                                <?= h($downtime['start']) ?>
                                -
                                <?= h($downtime['end']) ?>
                            </div>
                            <div class="col-3">
                                <?= h($downtime['duration_str']) ?>
                            </div>
                            <div class="col-5">
                                <?= h($downtime['comment_data']) ?>
                            </div>
                        </div>
                    <?php endforeach; ?>
                    <div class="row padding-bottom-10" style="background: #fbfbfb!important;"></div>
                <?php endif; ?>
            <?php endif; ?>
            <?php foreach ($host['Services'] as $service): ?>
                <?php if (empty($service['Service']['state_overview'])):
                    continue;
                endif ?>
                <div class="row border-top padding-top-10">
                    <div class="col-3 text-left">
                        <span class="padding-left-10" style=" word-break: break-word !important;
  overflow-wrap: break-word !important;">
                            <i class="fas fa-cog"></i>
                            <?= h($service['Service']['name']); ?>
                        </span>
                    </div>

                    <div class="col-2">
                        <div class="report-border-sucess padding-left-2">
                            <?php if ($service['Service']['configuration_option'] & 1): ?>
                                <div class="row">
                                    <div class="col-12">
                                        <!-- 1 == percent -->
                                        <?= round($service['Service']['state_overview']['percent_ok'], 3) ?> %
                                    </div>
                                </div>
                            <?php endif; ?>
                            <?php if ($service['Service']['configuration_option'] & 2): ?>
                                <div class="row">
                                    <div class="col-12">
                                        <!-- 2 == minute -->
                                        <?= h($service['Service']['state_overview']['human_ok']) ?>
                                    </div>
                                </div>
                            <?php endif; ?>
                        </div>
                    </div>

                    <div class="col-2">
                        <div class="report-border-warning padding-left-2">
                            <?php if ($service['Service']['configuration_option'] & 1): ?>
                                <div class="row">
                                    <div class="col-12">
                                        <!-- 1 == percent -->
                                        <?= round($service['Service']['state_overview']['percent_warning'], 3) ?> %
                                    </div>
                                </div>
                            <?php endif; ?>
                            <?php if ($service['Service']['configuration_option'] & 2): ?>
                                <div class="row">
                                    <div class="col-12">
                                        <!-- 2 == minute -->
                                        <?= h($service['Service']['state_overview']['human_warning']) ?>
                                    </div>
                                </div>
                            <?php endif; ?>
                        </div>
                    </div>

                    <div class="col-2">
                        <div class="report-border-danger padding-left-2">
                            <?php if ($service['Service']['configuration_option'] & 1): ?>
                                <div class="row">
                                    <div class="col-12">
                                        <!-- 1 == percent -->
                                        <?= round($service['Service']['state_overview']['percent_critical'], 3) ?> %
                                    </div>
                                </div>
                            <?php endif; ?>
                            <?php if ($service['Service']['configuration_option'] & 2): ?>
                                <div class="row">
                                    <div class="col-12">
                                        <!-- 2 == minute -->
                                        <?= h($service['Service']['state_overview']['human_critical']) ?>
                                    </div>
                                </div>
                            <?php endif; ?>
                        </div>
                    </div>

                    <div class="col-2">
                        <div class="report-border-default padding-left-2">
                            <?php if ($service['Service']['configuration_option'] & 1): ?>
                                <div class="row">
                                    <div class="col-12">
                                        <!-- 1 == percent -->
                                        <?= round($service['Service']['state_overview']['percent_unknown'], 3) ?> %
                                    </div>
                                </div>
                            <?php endif; ?>
                            <?php if ($service['Service']['configuration_option'] & 2): ?>
                                <div class="row">
                                    <div class="col-12">
                                        <!-- 2 == minute -->
                                        <?= h($service['Service']['state_overview']['human_unknown']) ?>
                                    </div>
                                </div>
                            <?php endif; ?>
                        </div>
                    </div>

                    <div class="col-1">
                        <!-- empty space -->
                    </div>
                </div>
                <div class="row padding-bottom-10"></div>

                <!-- Service Graph -->
                <?php if (!empty($service['Service']['state_overview']) && isset($report['GraphImageBlobs'][$host['Host']['uuid']][$service['Service']['uuid']])): ?>
                    <?php foreach ($report['GraphImageBlobs'][$host['Host']['uuid']][$service['Service']['uuid']] as $graphBase64Blob): ?>
                        <div class="row">
                            <div class="col-12 text-center">
                                <img
                                        src="data:image/png;base64,<?= $graphBase64Blob ?>"
                                        alt="Missing chart image">
                            </div>
                        </div>
                    <?php endforeach; ?>
                    <div class="row padding-bottom-10"></div>
                <?php endif; ?>

                <!-- Outages -->
                <?php if (sizeof($service['Service']['frontend']['outages']) > 0): ?>
                    <div class="row padding-top-5">
                        <!-- service outages -->
                        <div class="col-3"><!-- spacer --></div>
                        <div class="col-4">
                        </div>
                        <div class="col-5">
                            <?php if ($service['Service']['outage_duration'] !== null): ?>
                                <b><?= __('Outage time exceeding the permitted limit ({0} min)', $service['Service']['outage_duration']) ?></b>
                            <?php endif; ?>
                        </div>
                    </div>

                    <?php foreach ($service['Service']['frontend']['outages'] as $outage): ?>
                        <div class="row" style="padding: 0;">
                            <?php
                            $style = '';
                            if ($outage['duration'] === $service['Service']['maximum_outage_duration']):
                                $style = 'border: 1px solid #DEE3E5!important; background: #DEE3E5!important;';
                            endif;
                            ?>
                            <div class="col-3" style="<?= $style ?>"><!-- spacer --></div>
                            <div class="col-4" style="<?= $style ?>">
                                <?= h($outage['start']) ?>
                                -
                                <?= h($outage['end']) ?>
                            </div>
                            <div class="col-5" style="<?= $style ?>">
                                <?= h($outage['duration_str']) ?>
                            </div>
                        </div>
                    <?php endforeach; ?>
                    <div class="row padding-bottom-10"></div>
                <?php endif; ?>

                <!-- Outages in downtimes -->
                <?php if (sizeof($service['Service']['frontend']['outages_summarized_in_downtime']) > 0): ?>
                    <div class="row padding-top-5">
                        <!-- host outages -->
                        <div class="col-3"><!-- spacer --></div>
                        <div class="col-9">
                            <b>
                                <i class="far fa-clock"></i>
                                <?php if ($service['Service']['outage_duration']): ?>
                                    <?= __('Service outage in downtime exceeding the permitted limit ({0} min)', $service['Service']['outage_duration']) ?>
                                <?php else: ?>
                                    <?= __('Service outage in downtime') ?>
                                <?php endif; ?>
                            </b>
                        </div>
                    </div>

                    <?php foreach ($service['Service']['frontend']['outages_summarized_in_downtime'] as $outage): ?>
                        <div class="row" style="padding: 0;">
                            <div class="col-3"><!-- spacer --></div>
                            <div class="col-4">
                                <?= h($outage['start']) ?>
                                -
                                <?= h($outage['end']) ?>
                            </div>
                            <div class="col-5">
                                <?= h($outage['duration_str']) ?>
                            </div>
                        </div>
                    <?php endforeach; ?>
                    <div class="row padding-bottom-10"></div>
                <?php endif; ?>

                <!-- Service downtimes -->
                <?php if (sizeof($service['Service']['frontend']['downtimes']) > 0): ?>
                    <div class="row html-report-host-row padding-top-5">
                        <!-- host outages -->
                        <div class="col-3"><!-- spacer --></div>
                        <div class="col-9">
                            <b>
                                <i class="fas fa-power-off"></i>
                                <?= __('Service downtimes') ?>
                            </b>
                        </div>
                    </div>

                    <?php foreach ($service['Service']['frontend']['downtimes'] as $downtime): ?>
                        <div class="row" style="padding: 0;">
                            <div class="col-3"><!-- spacer --></div>
                            <div class="col-4">
                                <?= h($downtime['start']) ?>
                                -
                                <?= h($downtime['end']) ?>
                            </div>
                            <div class="col-3">
                                <?= h($downtime['duration_str']) ?>
                            </div>
                            <div class="col-5">
                                <?= h($downtime['comment_data']) ?>
                            </div>
                        </div>
                    <?php endforeach; ?>
                    <div class="row padding-bottom-10"></div>
                <?php endif; ?>

            <?php endforeach; ?>
        <?php endforeach; ?>
    </div>
</div>
