angular.module('openITCOCKPIT')
    .controller('AutoreportsAddStepTwoController', function($scope, $state, $stateParams, $http, NotyService, RedirectService, QueryStringService){
        $scope.post = {
            Autoreport: {
                hosts: {},
                services: {}
            }
        };

        $scope.selectedHostIds = [];
        $scope.selectedSelectedIds = [];
        $scope.servicenameRegex = '';
        $scope.serviceIdsToConsider = [];

        $scope.defaultOptionsPercent = 0;
        $scope.defaultOptionsMinute = 0;
        $scope.defaultOptionsAliasGraph = 0;
        $scope.defaultOptionsGraphSettings = 'AVERAGE';
        $scope.defaultOptionsOutageDuration = '';
        $scope.defaultOptionsAllfailures = 0;

        $scope.init = true;
        $scope.id = $stateParams.id;
        $scope.autoreport = null;
        $scope.hostsWithServices = {};

        var updateDefaultOption = function(field, value){
            for(var hostId in $scope.post.Autoreport.hosts){
                if(field === 'graphSettings'){
                    continue;
                }

                $scope.post.Autoreport.hosts[hostId][field] = value;
            }

            for(var serviceId in $scope.post.Autoreport.services){
                if(field === 'alias'){
                    $scope.post.Autoreport.services[serviceId].graph = value;
                }else{
                    $scope.post.Autoreport.services[serviceId][field] = value;
                }
            }
        };

        var cleanUpForServiceOptions = function(serviceIdsToDelete){
            if(serviceIdsToDelete.length === 0){
                return;
            }
            for(var index in serviceIdsToDelete){
                var serviceId = serviceIdsToDelete[index];
                if($scope.post.Autoreport.services.hasOwnProperty(serviceId)){
                    delete $scope.post.Autoreport.services[serviceId];
                }
            }
        };


        $scope.load = function(){
            var params = {
                'angular': true
            };

            $http.get("/autoreport_module/autoreports/addStepTwo/" + $scope.id + ".json", {
                params: params
            }).then(function(result){
                $scope.autoreport = result.data.autoreport;
                $scope.init = false;
                $scope.loadHosts('');
            }, function errorCallback(result){
                if(result.status === 403){
                    $state.go('403');
                }

                if(result.status === 404){
                    $state.go('404');
                }
            });
        };


        $scope.loadHosts = function(searchString){
            $http.get("/hosts/loadHostsByContainerId.json", {
                params: {
                    'angular': true,
                    'containerId': $scope.autoreport.container_id,
                    'filter[Hosts.name]': searchString,
                    'selected[]': $scope.selectedHostIds,
                    'resolveContainerIds': true
                }
            }).then(function(result){
                $scope.hosts = result.data.hosts;
            });
        };

        $scope.loadServicesWithHostByHostIds = function(){
            for(var index in $scope.post.Autoreport.services){
                if($scope.post.Autoreport.services[index]['percent'] || $scope.post.Autoreport.services[index]['minute']){
                    if(!in_array($scope.post.Autoreport.services[index]['service_id'], $scope.selectedSelectedIds)){
                        $scope.selectedSelectedIds.push($scope.post.Autoreport.services[index]['service_id']);
                    }
                }
            }
            $http.get("/autoreport_module/autoreports/hostAndServicesByHostId.json", {
                params: {
                    'angular': true,
                    'hostIds[]': $scope.selectedHostIds,
                    'servicenameRegex': $scope.servicenameRegex,
                    'selected[]': $scope.selectedSelectedIds
                }
            }).then(function(result){
                $scope.hostsWithServices = result.data.hostsWithServices;
                $scope.serviceIdsToConsider = result.data.serviceIdsList;


                for(var hostIndex in result.data.hostsWithServices){
                    var hostId = result.data.hostsWithServices[hostIndex].id;

                    //Do not overwrite already selected host report options
                    if(!$scope.post.Autoreport.hosts.hasOwnProperty(hostId)){
                        $scope.post.Autoreport.hosts[hostId] = {
                            host_id: hostId,
                            percent: 0,
                            minute: 0,
                            alias: 0,
                            outage: '',
                            allfailures: 0
                        };
                    }

                    for(var serviceIndex in result.data.hostsWithServices[hostIndex].services){
                        var serviceId = result.data.hostsWithServices[hostIndex].services[serviceIndex].id;

                        //Do not overwrite already selected service report options
                        if(!$scope.post.Autoreport.services.hasOwnProperty(serviceId)){
                            $scope.post.Autoreport.services[serviceId] = {
                                host_id: hostId,
                                service_id: serviceId,
                                percent: 0,
                                minute: 0,
                                graph: 0,
                                graphSettings: 'AVERAGE',
                                outage: '',
                                allfailures: 0
                            };
                        }
                    }
                    var idsToDelete = _.difference(_.map(_.keys($scope.post.Autoreport.services), Number), $scope.serviceIdsToConsider);
                    cleanUpForServiceOptions(idsToDelete);
                }

            });
        };

        $scope.submit = function(){
            $http.post("/autoreport_module/autoreports/addStepTwo/" + $scope.id + ".json?angular=true",
                $scope.post
            ).then(function(result){
                $scope.errors = {};
                $state.go('AutoreportsEditStepThree', {
                    id: $scope.id
                }).then(function(){
                    NotyService.scrollTop();
                });

            }, function errorCallback(result){
                if(result.data.hasOwnProperty('error')){
                    $scope.errors = result.data.error;
                    NotyService.genericError();
                }
            });
        };


        $scope.$watch('selectedHostIds', function(){
            if($scope.init){
                return;
            }
            if($scope.selectedHostIds.length === 0){
                $scope.post.Autoreport.hosts = {};
                $scope.post.Autoreport.services = {};
                $scope.selectedSelectedIds = [];
                return;
            }else{
                var deletedHostIds = [];
                for(var hostId in $scope.post.Autoreport.hosts){
                    hostId = parseInt(hostId, 10);
                    if($scope.selectedHostIds.indexOf(hostId) === -1){
                        delete $scope.post.Autoreport.hosts[hostId];
                        deletedHostIds.push(hostId);
                    }
                }

                //Also delete services of deleted hosts
                if(deletedHostIds.length > 0){
                    for(serviceId in $scope.post.Autoreport.services){
                        hostId = parseInt($scope.post.Autoreport.services[serviceId].host_id, 10);
                        if($scope.selectedHostIds.indexOf(hostId) === -1){
                            //host was removed from selectbox
                            delete $scope.post.Autoreport.services[serviceId];
                        }
                    }
                }
            }

            $scope.loadServicesWithHostByHostIds('');
        }, true);

        $scope.$watch('post.Autoreport.services', function(newValues, oldValues){
            if($scope.init){
                return;
            }
            if(angular.equals(newValues, oldValues)){
                return;
            }
            $scope.selectedSelectedIds = [];
            for(var index in newValues){
                if(newValues[index]['percent'] || newValues[index]['minute']){
                    if(!in_array(newValues[index]['service_id'], $scope.selectedSelectedIds)){
                        $scope.selectedSelectedIds.push($scope.post.Autoreport.services[index]['service_id']);
                    }
                }
            }
        }, true);

        $scope.$watch('servicenameRegex', function(){
            if($scope.init){
                return;
            }
            if($scope.selectedHostIds.length === 0){
                return;
            }
            $scope.loadServicesWithHostByHostIds('');
        }, true);

        $scope.$watch('defaultOptionsPercent', function(){
            if($scope.init){
                return;
            }

            updateDefaultOption('percent', $scope.defaultOptionsPercent);
        });

        $scope.$watch('defaultOptionsMinute', function(){
            if($scope.init){
                return;
            }
            updateDefaultOption('minute', $scope.defaultOptionsMinute);
        });

        $scope.$watch('defaultOptionsAliasGraph', function(){
            if($scope.init){
                return;
            }
            updateDefaultOption('alias', $scope.defaultOptionsAliasGraph);
        });

        $scope.$watch('defaultOptionsGraphSettings', function(){
            if($scope.init){
                return;
            }
            updateDefaultOption('graphSettings', $scope.defaultOptionsGraphSettings);
        });

        $scope.$watch('defaultOptionsOutageDuration', function(){
            if($scope.init){
                return;
            }
            updateDefaultOption('outage', $scope.defaultOptionsOutageDuration);
        });

        $scope.$watch('defaultOptionsAllfailures', function(){
            if($scope.init){
                return;
            }
            updateDefaultOption('allfailures', $scope.defaultOptionsAllfailures);
        });

        $scope.load();

    });
