angular.module('openITCOCKPIT')
    .controller('AutoreportsIndexController', function($scope, $http, SortService, MassChangeService, $state){

        SortService.setSort('Autoreports.name');
        SortService.setDirection('asc');
        $scope.currentPage = 1;
        $scope.useScroll = true;

        /*** Filter Settings ***/
        var defaultFilter = function(){
            $scope.filter = {
                Autoreport: {
                    name: '',
                    description: ''
                }
            };
        };
        /*** Filter end ***/

        $scope.massChange = {};
        $scope.selectedElements = 0;
        $scope.deleteUrl = '/autoreport_module/autoreports/delete/';

        $scope.showFilter = false;

        $scope.load = function(){
            $http.get("/autoreport_module/autoreports/index.json", {
                params: {
                    'angular': true,
                    'scroll': $scope.useScroll,
                    'sort': SortService.getSort(),
                    'page': $scope.currentPage,
                    'direction': SortService.getDirection(),
                    'filter[Autoreports.name]': $scope.filter.Autoreport.name,
                    'filter[Autoreports.description]': $scope.filter.Autoreport.description
                }
            }).then(function(result){
                $scope.autoreports = result.data.autoreports;
                $scope.paging = result.data.paging;
                $scope.scroll = result.data.scroll;
                $scope.init = false;

            }, function errorCallback(result){
                if(result.status === 403){
                    $state.go('403');
                }

                if(result.status === 404){
                    $state.go('404');
                }
            });
        };


        $scope.triggerFilter = function(){
            $scope.showFilter = !$scope.showFilter === true;
        };

        $scope.resetFilter = function(){
            defaultFilter();
            $scope.undoSelection();
        };

        $scope.undoSelection = function(){
            MassChangeService.clearSelection();
            $scope.massChange = MassChangeService.getSelected();
            $scope.selectedElements = MassChangeService.getCount();
        };

        $scope.selectAll = function(){
            if($scope.autoreports){
                for(var key in $scope.autoreports){
                    if($scope.autoreports[key].allowEdit){
                        var id = $scope.autoreports[key].id;
                        $scope.massChange[id] = true;
                    }
                }
            }
        };

        $scope.getObjectsForDelete = function(){
            var objects = {};
            var selectedObjects = MassChangeService.getSelected();
            for(var key in $scope.autoreports){
                for(var id in selectedObjects){
                    if(id == $scope.autoreports[key].id){
                        objects[id] = $scope.autoreports[key].name;
                    }
                }
            }
            return objects;
        };

        $scope.getObjectForDelete = function(Autoreport){
            var object = {};
            object[Autoreport.id] = Autoreport.name;
            return object;
        };

        $scope.changepage = function(page){
            if(page !== $scope.currentPage){
                $scope.currentPage = page;
                $scope.load();
            }
        };

        $scope.changeMode = function(val){
            $scope.useScroll = val;
            $scope.load();
        };

        $scope.$watch('filter', function(){
            $scope.currentPage = 1;
            $scope.undoSelection();
            $scope.load();
        }, true);

        $scope.$watch('massChange', function(){
            MassChangeService.setSelected($scope.massChange);
            $scope.selectedElements = MassChangeService.getCount();
        }, true);

        defaultFilter();
        SortService.setCallback($scope.load);

    });
