angular.module('openITCOCKPIT').directive('autoreportWidget', function($http){
    return {
        restrict: 'E',
        templateUrl: '/autoreport_module/autoreports/autoreportWidget.html',
        scope: {
            'widget': '='
        },

        controller: function($scope, $sce){
            /** public vars **/
            $scope.init = true;
            $scope.autoreportConfig = {
                id: null,
                in_percent: true,
                set_color_dynamically: false,
                show_update_date: true,
                show_bar_chart: true
            };

            var $widget = $('#widget-' + $scope.widget.id);

            $scope.frontWidgetHeight = parseInt(($widget.height() - 70), 10); //-50px header
            $scope.frontWidgetWidth = parseInt(($widget.width() - 22), 10); //-12px padding from grid + 5*2 px padding from widget
            $scope.backgroundColor = '#4285F4';
            $scope.autoreportAvailabilityLogs = [];

            $widget.on('resize', function(){
                if($scope.init){
                    return;
                }
                hasResize();
            });

            $scope.autoreportOverviewTimeout = null;

            $scope.load = function(){
                $http.get("/autoreport_module/autoreports/autoreportWidget.json", {
                    params: {
                        'angular': true,
                        'widgetId': $scope.widget.id
                    }
                }).then(function(result){
                    $scope.autoreport = result.data.autoreport;
                    $scope.autoreportConfig = result.data.config;
                    $scope.autoreportConfig.id = parseInt(result.data.config.Autoreport.id, 10);
                    if($scope.autoreportConfig.Autoreport.set_color_dynamically && $scope.autoreport.last_percent_value){
                        $scope.backgroundColor = getBackgroundColor(
                            $scope.autoreport.last_percent_value,
                            $scope.autoreport.min_availability
                        );
                    }else{
                        $scope.backgroundColor = '#4285F4';
                    }
                    if($scope.autoreport.id !== null){
                        $scope.autoreportAvailabilityLogs = $scope.autoreport.autoreport_availability_log;
                    }
                    $scope.fontSize = parseInt($scope.frontWidgetHeight / 8.5, 10);
                    if($scope.autoreportConfig.Autoreport.in_percent){
                        $scope.fontSize = parseInt($scope.frontWidgetHeight / 4.5, 10);
                    }
                    $scope.ACL = result.data.ACL;
                    $scope.autoreportHref = getHref();
                    //Do not trigger watch on page load
                    setTimeout(function(){
                        $scope.init = false;
                        if($scope.autoreportConfig.Autoreport.show_bar_chart && $scope.autoreportAvailabilityLogs.length > 0){
                            createSparklineGraph($scope.autoreportConfig.Autoreport.in_percent, $scope.autoreportAvailabilityLogs);
                        }
                    }, 250);
                });
            };

            $scope.hideConfig = function(){
                $scope.$broadcast('FLIP_EVENT_IN');
                $scope.load();
            };
            $scope.showConfig = function(){
                $scope.$broadcast('FLIP_EVENT_OUT');
                $scope.loadAutoreports();
            };
            $scope.calculateAvailabilityForReport = function(reportId){
                $http.get("/autoreport_module/autoreports/calculateAvailability/" + reportId + ".json?angular=true").then(function(result){
                    $scope.load();
                    return true;
                });
            };

            var hasResize = function(){
                if($scope.init){
                    return;
                }
                $scope.frontWidgetHeight = parseInt(($widget.height() - 70), 10); //-50px header
                $scope.frontWidgetWidth = parseInt(($widget.width() - 22), 10); //-12px padding from grid + 5*2 px padding from widget

                $scope.fontSize = parseInt($scope.frontWidgetHeight / 8.5, 10);

                if($scope.autoreportConfig.Autoreport.in_percent){
                    $scope.fontSize = parseInt($scope.frontWidgetHeight / 4.5, 10);
                }

                if($scope.autoreportOverviewTimeout){
                    clearTimeout($scope.autoreportOverviewTimeout);
                }
                $scope.autoreportOverviewTimeout = setTimeout(function(){
                    $scope.load();
                }, 500);
            };

            var getBackgroundColor = function getColor(value, minimalAvailability){
                //value from 0 to 100
                var colorLightness = 40;
                var hue = 120;
                if(value < 100 && value < minimalAvailability){
                    hue = 0;
                }else if(value < 100 && value >= minimalAvailability){
                    hue = parseInt(((value - minimalAvailability) / (100 - minimalAvailability)) * 120, 10);
                    if(hue > 120){
                        hue = 120;
                    }
                }
                return ['hsl(', hue, ',100%,' + colorLightness + '%)'].join('');
            };

            var createSparklineGraph = function(inPercent, autoreportAvailabilityLogs){
                var barSpacing = 3;
                var valueCount = 25; //show last 20 log entries
                var barWidth = Math.round(($scope.frontWidgetWidth - (valueCount - 1) * barSpacing) / valueCount);
                var sparkLineMaxValue = [':' + $scope.autoreport.min_availability];
                var sparkLineColorMap = {};

                if(inPercent){
                    sparkLineMaxValue.map(function(v){
                        sparkLineColorMap['"' + v + '"'] = '#f99d9d';
                    });
                }

                var logPoints = [];
                var logDates = [];
                var tooltipValues = [];
                var maxValueInMinutes = 0;

                for(var key in autoreportAvailabilityLogs){
                    if(inPercent){
                        logPoints.push(autoreportAvailabilityLogs[key]['determined_availability_percent']);
                        tooltipValues.push(autoreportAvailabilityLogs[key]['determined_availability_percent'] + '%');
                    }else{
                        logPoints.push(autoreportAvailabilityLogs[key]['determined_availability_time']);
                        if(maxValueInMinutes < autoreportAvailabilityLogs[key]['determined_availability_time']){
                            maxValueInMinutes = autoreportAvailabilityLogs[key]['determined_availability_time'];
                        }
                        tooltipValues.push(autoreportAvailabilityLogs[key]['determined_availability_time_human'])
                    }
                    logDates.push(autoreportAvailabilityLogs[key]['last_update_human']);
                }
                var sparklineOptions = {
                    type: "bar",
                    barSpacing: barSpacing,
                    barWidth: barWidth,
                    chartRangeMin: 0,
                    height: 35,
                    barColor: "white",
                    negBarColor: '#f44',
                    colorMap: sparkLineColorMap,
                    tooltipFormat: '<span class="label label-default font-sm chart-label">' +
                        '{{value}}</span> <i class="fa fa-calendar padding-left-5"></i> {{offset:log_date}}' +
                        ' - {{offset:log_value}}',
                    tooltipValueLookups: {
                        'log_date': logDates,
                        'log_value': tooltipValues
                    }
                };

                if(inPercent){
                    sparklineOptions.chartRangeMax = 100;
                    sparklineOptions.tooltipFormat = '<span class="label label-default font-sm chart-label">' +
                        '{{value}}%</span> <i class="fa fa-calendar padding-left-5"></i> {{offset:log_date}}';
                }else{
                    sparklineOptions.chartRangeMax = maxValueInMinutes;
                    sparklineOptions.tooltipFormat = '<span class="label label-default font-sm chart-label">' +
                        '{{offset:log_value}}</span> <i class="fa fa-calendar padding-left-5"></i> {{offset:log_date}}';
                }

                $('#inlinesparkline-' + $scope.widget.id).sparkline(logPoints, sparklineOptions
                );
            };

            $scope.loadAutoreports = function(){
                $http.get("/autoreport_module/autoreports/loadAutoreports.json", {
                    params: {
                        'angular': true,
                        'availabilityLog': true
                    }
                }).then(function(result){
                    $scope.autoreports = result.data.autoreports;
                });
            };


            $scope.saveAutoreportWidget = function(){
                $http.post("/autoreport_module/autoreports/autoreportWidget.json?angular=true",
                    {
                        Widget: {
                            id: $scope.widget.id
                        },
                        Autoreport: {
                            id: $scope.autoreportConfig.id,
                            in_percent: $scope.autoreportConfig.Autoreport.in_percent,
                            set_color_dynamically: $scope.autoreportConfig.Autoreport.set_color_dynamically,
                            show_update_date: $scope.autoreportConfig.Autoreport.show_update_date,
                            show_bar_chart: $scope.autoreportConfig.Autoreport.show_bar_chart
                        },
                    }
                ).then(function(result){
                    //Update status
                    $scope.hideConfig();
                });
            };

            var getHref = function(){
                var url = 'javascript:void(0);';
                if($scope.ACL.autoreport.generate){
                    return '/autoreport_module/autoreports/generate/' + $scope.autoreportConfig.id
                }

                return url;
            };

            /** Page load / widget get loaded **/
            $scope.load();
        },

        link: function($scope, element, attr){

        }
    };
});
